#! /usr/bin/python
# -*- coding: utf-8 -*-

# import funkcí z jiného adresáře
import logging
import os.path
import sys
import unittest

logger = logging.getLogger(__name__)

from nose.plugins.attrib import attr
import numpy as np
import shutil

import matplotlib.pyplot as plt

path_to_script = os.path.dirname(os.path.abspath(__file__))
# sys.path.append(os.path.join(path_to_script, "../"))
# sys.path.append(os.path.join(path_to_script, "../lisa/"))

import lisa.volumetry_evaluation as ve
import lisa
import lisa.experiments
import lisa.dataset
from lisa.dataset import join_sdp


# just test if everything is going ok
def speceval(vol1, vol2, vs):
    """
    special_evaluation_function
    """
    return {'one': 1}

class ExperimentsTest(unittest.TestCase):

    def test_experiment_set_(self):
        """
        pklz_dirs as prefix is tested.
        """

# if directory exists, remove it
        experiment_dir = join_sdp("tmp_tests/tmp_exp_small2/")
        experiment_name = 'exp2'

        if os.path.exists(experiment_dir):
            shutil.rmtree(experiment_dir)
        os.makedirs(experiment_dir)

        # experiment_dir = join_sdp("exp_small2/")

        sliver_reference_dir = join_sdp("exp_small/seg")

# this is setup for visualization
        markers = ['ks',
                   # 'r<'
                   ]
        labels = ['vs 6 mm',
                  'vs 7 mm'
                  ]
        input_data_path_pattern = join_sdp("exp_small/seeds/*.pklz")

        conf_default = {
            'config_version': [1, 0, 0], 'working_voxelsize_mm': 2.0,
            'segmentation_smoothing': False,
            'segparams': {'pairwise_alpha_per_mm2': 50,
                          'return_only_object_with_seeds': True}
        }
        conf_list = [
            {'working_voxelsize_mm': 6},
            {'working_voxelsize_mm': 7}
        ]

#
# # experiment_support.report(pklz_dirs, labels, markers)
        ramr = lisa.experiments.RunAndMakeReport(
            experiment_dir, labels, sliver_reference_dir,
            input_data_path_pattern,
            conf_default=conf_default,
            conf_list=conf_list,
            show=False, use_plt=False,
            experiment_name=experiment_name,
            markers=markers
        )
        ramr.config()
        dir_eval = os.listdir(experiment_dir)
        # self.assertIn('exp1_eval.pkl', dir_eval)
        # self.assertIn('exp1_eval.csv', dir_eval)
        self.assertIn('exp2-vs6mm.config', dir_eval)
        self.assertIn('exp2-vs7mm.config', dir_eval)
        # self.assertIn('exp1.yaml', dir_eval)
#         # self.assertTrue(False)
        ramr.run_experiments()
        ramr.evaluation()
        ramr.report()
        # shutil.rmtree(dire)

    @attr("slow")
    def test_experiment_set(self):
        import lisa.experiments

        experiment_path = join_sdp("exp")
        # this var is no necessary
        pklz_dirs = [
            join_sdp("exp/exp1"),
            join_sdp("exp/exp2"),
            # "/home/mjirik/projects/lisa/sample_data/exp1",
            # "/home/mjirik/projects/lisa/sample_data/exp2",

        ]
        sliver_reference_dir = join_sdp("exp/seg")
        # "/home/mjirik/data/medical/orig/sliver07/training/"

# this is setup for visualization
        markers = ['ks', 'r<']
        labels = ['3gaus', '02smoothing']
        input_data_path_pattern = join_sdp("exp/seeds/*.pklz")

# if directory exists, remove it
        for dire in pklz_dirs:
            if os.path.exists(dire):
                shutil.rmtree(dire)

# experiment_support.report(pklz_dirs, labels, markers)
        lisa.experiments.run_and_make_report(
            experiment_path, labels, sliver_reference_dir,
            input_data_path_pattern, markers=markers, pklz_dirs=pklz_dirs, show=False)
        import io3d.misc
        obj = io3d.misc.obj_from_file(pklz_dirs[0] + '.yaml', filetype='yaml')
        self.assertGreater(len(obj['data']), 0)
        # self.assertTrue(False)

    # not sure why is this test failing on travis. On local there are no
    # problems.  Maybe it is not called.
    @attr("incomplete")
    @attr("slow")
    def test_experiment_set_small(self):
        import lisa.experiments

        # os.path.join(path_to_script, "..")
        # pklz_dirs = [
        #     join_sdp("exp_small/exp1"),
        #     # join_sdp("exp_small/exp2"),
        # ]
        experiment_dir = join_sdp("tmp_tests/exp1")
        sliver_reference_dir = join_sdp("exp_small/seg")
        # "/home/mjirik/data/medical/orig/sliver07/training/"

# this is setup for visualization
        markers = ['ks',
                   # 'r<'
                   ]
        labels = ['vs6mm',
                  # '02smoothing'
                  ]
        input_data_path_pattern = join_sdp("exp_small/seeds/*.pklz")

        conf_default = {
            'config_version': [1, 0, 0], 'working_voxelsize_mm': 2.0,
            'segmentation_smoothing': False,
            'segparams': {'pairwise_alpha_per_mm2': 50,
                          'return_only_object_with_seeds': True}
        }
        conf_list = [
            # {'working_voxelsize_mm': 4},
            {'working_voxelsize_mm': 6}
        ]

# if directory exists, remove it
        if os.path.exists(experiment_dir):
            shutil.rmtree(experiment_dir)

# experiment_support.report(pklz_dirs, labels, markers)
        lisa.experiments.run_and_make_report(
            experiment_dir, labels, sliver_reference_dir,
            input_data_path_pattern,
            conf_default=conf_default,
            conf_list=conf_list,
            show=False,
            markers=markers
        )
        import io3d.misc
        path_to_yaml = join_sdp("tests_tmp/exp1/exp1-vs6mm.yaml")
        obj = io3d.misc.obj_from_file(path_to_yaml, filetype='yaml')
        self.assertGreater(len(obj['data']), 0)
        # self.assertTrue(False)

    @attr("actual")
    # @unittest.skip("this test is little rebel under travis-ci")
    def test_experiment_set_small_per_partes(self):
        plt.ioff()
        # import lisa.experiments

        # os.path.join(path_to_script, "..")
        experiment_dir = lisa.dataset.join_sdp("exp22")

        sliver_reference_dir = join_sdp("exp_small/seg")
        # "/home/mjirik/data/medical/orig/sliver07/training/"

# this is setup for visualization
        labels = ['vs6mm',
                  # '02smoothing'
                  ]
        input_data_path_pattern = lisa.dataset.join_sdp("exp_small/seeds/*.pklz")

        conf_default = {
            'config_version': [1, 0, 0], 'working_voxelsize_mm': 2.0,
            'segmentation_smoothing': False,
            "run_organ_segmentation": True,
            'segparams': {'pairwise_alpha_per_mm2': 50,
                          'return_only_object_with_seeds': True}
        }
        conf_list = [
            # {'working_voxelsize_mm': 4},
            {'working_voxelsize_mm': 6}
        ]

        if os.path.exists(experiment_dir):
            shutil.rmtree(experiment_dir)
        os.mkdir(experiment_dir)
#
# # experiment_support.report(pklz_dirs, labels, markers)
        ramr = lisa.experiments.RunAndMakeReport(
            experiment_dir, labels, sliver_reference_dir,
            input_data_path_pattern,
            conf_default=conf_default,
            conf_list=conf_list,
            show=False, use_plt=False)

        self.assertTrue(ramr.is_evaluation_necessary())
        self.assertTrue(ramr.is_run_experiments_necessary())


        ramr.config()
        ramr.run_experiments()
        # ramr.evaluation(special_evaluation_function=speceval)
        ramr.evaluation(
            special_evaluation_function=lisa.volumetry_evaluation.compare_volumes_boundingbox)
        ramr.report()

        self.assertFalse(ramr.is_evaluation_necessary())
        self.assertFalse(ramr.is_run_experiments_necessary())

        data_path = lisa.dataset.join_sdp("exp22/")
        dir_eval = os.listdir(data_path)
        self.assertIn('exp22-vs6mm_eval.pkl', dir_eval)
        self.assertIn('exp22-vs6mm_eval.csv', dir_eval)
        self.assertIn('exp22-vs6mm.config', dir_eval)
        self.assertIn('exp22-vs6mm.yaml', dir_eval)
        import io3d.misc
        obj = io3d.misc.obj_from_file(
            os.path.join(experiment_dir, 'exp22-vs6mm.yaml'),
            filetype='yaml')
        self.assertGreater(len(obj['data']), 0)
#         # self.assertTrue(False)

    def prepare_data_for_fast_experiment(self):
        """
        not used
        """

        pklz_dirs = [
            lisa.dataset.join_sdp("exp_synth/exp1"),
            lisa.dataset.join_sdp("exp_synth/exp2")
            ]

        os.mkdir(
            lisa.dataset.join_sdp("exp_synth"),
        )
        os.mkdir(pklz_dirs[0])
        os.mkdir(pklz_dirs[1])
        pass

    def test_get_subdirs(self):
        dirpath = os.path.join(path_to_script, "..")
        dirlist = lisa.experiments.get_subdirs(dirpath)
        # import pdb; pdb.set_trace()

        self.assertTrue('tests' in dirlist)
        self.assertTrue('lisa' in dirlist)
        self.assertFalse('README.md' in dirlist)

    def test_sliver_overall_score_for_one_couple(self):

        score = {
            'vd': 15.5,
            'voe': 15.6,
            'avgd': 14.4,
            'rmsd': 20,
            'maxd': 10,
        }

        overall_score = ve.sliver_overall_score_for_one_couple(score)
        self.assertAlmostEqual(15.1, overall_score)

    def test_eval_sliver_matrics(self):
        voxelsize_mm = [1, 2, 3]

        vol1 = np.zeros([20, 21, 22], dtype=np.int8)
        vol1[10:15, 10:15, 10:15] = 1

        vol2 = np.zeros([20, 21, 22], dtype=np.int8)
        vol2[10:15, 10:16, 10:15] = 1

        eval1 = ve.compare_volumes(vol1, vol2, voxelsize_mm)
        # print ve.sliver_score(eval1['vd'], 'vd')

        self.assertAlmostEqual(eval1['vd'], 20.0)

        score = ve.sliver_score_one_couple(eval1)
        # score is 21.875
        self.assertGreater(score['vd'], 20)
        self.assertLess(score['vd'], 25)

    def test_eval_sliver_distance_for_two_pixels_bigger_volume(self):
        """
        maxd is measured in corner. It is  space diagonal of 2 pixels cube.
        """

        vol1 = np.zeros([20, 21, 22], dtype=np.int8)
        vol1[10:15, 10:15, 10:15] = 1

        vol2 = np.zeros([20, 21, 22], dtype=np.int8)
        vol2[8:17, 8:17, 8:17] = 1

        eval1 = ve.distance_matrics(vol1, vol2, [1, 1, 1])

        self.assertAlmostEquals(eval1[2], 3 ** (0.5) * 2)

    def test_eval_sliver_distance_two_points(self):
        """
        Two points 2 by 2 pixels diagonal.
        """

        vol1 = np.zeros([20, 21, 22], dtype=np.int8)
        vol1[10, 10, 10] = 1

        vol2 = np.zeros([20, 21, 22], dtype=np.int8)
        vol2[10, 12, 12] = 1

        eval1 = ve.distance_matrics(vol1, vol2, [1, 1, 1])
        self.assertAlmostEquals(eval1[2], np.sqrt(2) * 2)

    def test_eval_sliver_distance_two_points_non_comutative(self):
        """
        Two volumes - obj1 1px and obj2 2px
        There is different maximal distance from first to second then
        from second to first

        """

        vol1 = np.zeros([20, 21, 22], dtype=np.int8)
        vol1[10, 10, 10] = 1

        vol2 = np.zeros([20, 21, 22], dtype=np.int8)
        vol2[10, 12, 10] = 1
        vol2[10, 14, 10] = 1

        eval1 = ve.distance_matrics(vol1, vol2, [1, 1, 1])

        self.assertLess(eval1[0], 4)
        self.assertGreater(eval1[0], 2)

    def test_eval_sliver_distance_two_points_with_half_voxelsize(self):
        """
        Two points 2 by 2 pixels diagonal and voxelsize is 0.5
        """

        vol1 = np.zeros([20, 21, 22], dtype=np.int8)
        vol1[10, 10, 10] = 1

        vol2 = np.zeros([20, 21, 22], dtype=np.int8)
        vol2[10, 12, 12] = 1

        eval1 = ve.distance_matrics(vol1, vol2, [0.5, 0.5, 0.5])
        self.assertAlmostEquals(eval1[2], np.sqrt(2))

    @attr("incomplete")
    def test_compare_eval_sliver_distance(self):
        """
        comparison of two methods for surface distance computation
        Second implementation is obsolete
        """

        vol1 = np.zeros([20, 21, 22], dtype=np.int8)
        vol1[10:15, 10:15, 10:15] = 1

        vol2 = np.zeros([20, 21, 22], dtype=np.int8)
        vol2[8:17, 8:17, 8:17] = 1

        eval1 = ve.distance_matrics(vol1, vol2, [1, 1, 1])

        self.assertAlmostEquals(eval1[2], 3 ** (0.5) * 2)
        # import lisa.liver_segmentation_cerveny as ls
        # import nearpy
        # engine = nearpy.Engine(dim=3)
        # eval2 = ls.vzdalenosti(vol1, vol2, [1, 1, 1], engine)
        # self.assertLess(eval1[0], 1.1 * eval2[0])
        # self.assertGreater(eval1[0], 0.9 * eval2[0])

    # @attr("incomplete")
    # def test_compare_eval_sliver_distance_bigger(self):
    #     """
    #     comparison of two methods for surface distance computation on bigger
    #     object
    #     Second implementation is obsolete
    #     """
    #
    #     vol1 = np.zeros([100, 210, 220], dtype=np.int8)
    #     vol1[10:50, 50:150, 100:150] = 1
    #
    #     vol2 = np.zeros([100, 210, 220], dtype=np.int8)
    #     vol2[12:52, 45:160, 100:150] = 1
    #
    #     eval1 = ve.distance_matrics(vol1, vol2, [1, 1, 1])
    #
    #     import lisa.liver_segmentation_cerveny as ls
    #     import nearpy
    #     engine = nearpy.Engine(dim=3)
    #     eval2 = ls.vzdalenosti(vol1, vol2, [1, 1, 1], engine)
    #     # import ipdb; ipdb.set_trace() #  noqa BREAKPOINT
    #     self.assertLess(eval1[0], 1.1 * eval2[0])
    #     self.assertGreater(eval1[0], 0.9 * eval2[0])

    def test_volumetry_evaluation_yaml_generator(self):
        """
        Test creates two directories and some empty files. Based on this
        structure a yaml file data are constructed.
        """
        import os
        import shutil

# Creating directory structure

        sliver_dir = '__test_sliver_dir'
        pklz_dir = '__test_pklz_dir'

        # d = os.path.dirname(sliver_dir)
        d = sliver_dir
        if os.path.exists(d):
            shutil.rmtree(d)
        # if not os.path.exists(d):
        os.makedirs(d)

        # d = os.path.dirname(pklz_dir)
        d = pklz_dir
        if os.path.exists(d):
            shutil.rmtree(d)
        # if not os.path.exists(d):
        os.makedirs(d)

        filelist1 = ['liver-seg001.mhd', 'liver-seg002.mhd',
                     'liver-seg006.mhd']
        for fl in filelist1:
            open(os.path.join(sliver_dir, fl), 'a').close()

        filelist2 = ['soubor_seg001to.pklz', 'soubor_seg002.pklz',
                     'sao_seg003.pklz', 'so_seg002tre3.pklz', 'ijij.pklz']
        for fl in filelist2:
            open(os.path.join(pklz_dir, fl), 'a').close()

# construct yaml data
        yamldata = ve.generate_input_yaml(
            sliver_dir, pklz_dir)

# assert

        fls0 = os.path.join(sliver_dir, filelist1[0])
        flp0 = os.path.join(pklz_dir, filelist2[0])
        # we only hope, that this will be first record
        self.assertEqual(yamldata['data'][0]['sliverseg'], fls0)
        self.assertEqual(yamldata['data'][0]['ourseg'], flp0)

# Clean
        # if os.path.exists(d):
        shutil.rmtree(sliver_dir)
        # if os.path.exists(d):
        shutil.rmtree(pklz_dir)

    def test_volumetry_evaluation_sliver_score(self):
        """
        Testing Volume Difference score. Score for negative values must be
        equal. Score for far high values must be 0.
        """
        score = ve.sliver_score([1, -1, 30, 50, 100], 'vd')
        self.assertAlmostEquals(score[0], score[1])
        self.assertEqual(score[2], 0)

if __name__ == "__main__":
    logging.basicConfig(stream=sys.stderr)
    logger.setLevel(logging.DEBUG)
    unittest.main()



#! /usr/bin/python
# -*- coding: utf-8 -*-



# import funkcí z jiného adresáře
import sys
import os.path

path_to_script = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.join(path_to_script, "../extern/pyseg_base/src/"))
sys.path.append(os.path.join(path_to_script, "../extern/sed3/"))
sys.path.append(os.path.join(path_to_script, "../src/"))
import unittest

from nose.plugins.attrib import attr
# import numpy as np


# import imcut.dcmreaddata as dcmr

from seededitorqt import QTSeedEditor

class QTSeedEditorTest(unittest.TestCase):
    interactive_tests = False

    @unittest.skipIf(not interactive_tests,"interactive test")
    def test_data_editor(self):
        """
        Funkce provádí změnu vstupních dat - data3d
        """
        #pyed = sed3.sed3(self.data3d, contour = oseg.segmentation)
        #pyed.show()

        from PyQt4.QtGui import QApplication
        import numpy as np
        im3d = np.random.rand(15,15,15)
        print("Select pixels for deletion (it will be setted to 0)")
#, QMainWindow
        app = QApplication(sys.argv)
        pyed = QTSeedEditor(im3d, mode='draw')
        pyed.exec_()


        deletemask = pyed.getSeeds()


        print("If it is ok, press 'Return'. If it is wrong, click into image and press 'Return'")
        # rewrite input data
        im3d [deletemask != 0] = 0
        pyed = QTSeedEditor(im3d)
        app.exec_()
        sds  = pyed.getSeeds()
# if usere select pixel, there will be 1 or 2 in sds
        self.assertLess(np.max(sds), 1)

    ##@unittest.skipIf(True,"interactive test")
    #def test_vincentka_06_slice_thickness_interactive(self):
    #    """
    #    Interactive test. SliceThickness is not voxel depth. If it is, this
    #    test will fail.
    #    """
    #    #dcmdir = os.path.join(path_to_script,'./../sample_data/matlab/examples/sample_data/DICOM/digest_article/')
    #    dcmdir = os.path.expanduser('~/data/medical/data_orig/vincentka/13021610/10200000/')
    #    dcmdir = os.path.expanduser('~/data/medical/data_orig/vincentka/13021610/12460000/')
    #    oseg = organ_segmentation.OrganSegmentation(dcmdir, working_voxelsize_mm = 4)
    #
# ma#nual seeds setting
    #    print("with left mouse button select some pixels of the bottle content")
    #    print("with right mouse button select some pixels of background")

    #    oseg.interactivity()

    #    volume = oseg.get_segmented_volume_size_mm3()
    #    #print volume

    #    self.assertGreater(volume,600000)
    #    self.assertLess(volume,850000)

    @attr("interactive")
    def test_data_editor_tree(self):
        """
        Just for visual check of seed edito
        """
        #pyed = sed3.sed3(self.data3d, contour = oseg.segmentation)
        #pyed.show()

        from PyQt4.QtGui import QApplication
        import numpy as np
        im3d = np.random.rand(15,15,15)
        print("Select pixels for deletion (it will be setted to 0)")
        #, QMainWindow
        app = QApplication(sys.argv)
        pyed = QTSeedEditor(im3d, mode='draw')
        pyed.exec_()


        deletemask = pyed.getSeeds()

if __name__ == "__main__":
    unittest.main()



#! /usr/bin/python
# -*- coding: utf-8 -*-

# import funkcí z jiného adresáře
import os.path
import os

path_to_script = os.path.dirname(os.path.abspath(__file__))
import unittest

import shutil


import numpy as np
from nose.plugins.attrib import attr

# import pydicom
# pydicom.debug(False)

#
import io3d.datawriter as dwriter
import io3d.datareader as dreader
import lisa.dataset
# import sed3 as pyed


class DicomWriterTest(unittest.TestCase):
    interactivetTest = False
#    def setUp(self):
# self.dcmdir = os.path.join(path_to_script,
# '../sample_data/jatra_06mm_jenjatraplus/')
#        self.dcmdir = os.path.join(path_to_script, '../sample_data/jatra_5mm')
# self.data3d, self.metadata = dcmr.dcm_read_from_dir(self.dcmdir)
#        reader = dcmr.DicomReader(self.dcmdir)
#        self.data3d = reader.get_3Ddata()
#        self.metadata = reader.get_metaData()

    def test_write_and_read(self):

        filename = 'test_file.dcm'
        data = (np.random.random([30, 100, 120]) * 30).astype(np.int16)
        data[0:5, 20:60, 60:70, ] += 30
        metadata = {'voxelsize_mm': [1, 2, 3]}
        dw = dwriter.DataWriter()
        dw.Write3DData(data, filename, filetype='dcm', metadata=metadata)

        dr = dreader.DataReader()
        newdata, newmetadata = dr.Get3DData(filename, dataplus_format=False)

        # print  "meta ", metadata
        # print  "new meta ", newmetadata
        # hack with -1024, because of wrong data reading
        self. assertEqual(data[10, 10, 10], newdata[10, 10, 10])
        self. assertEqual(data[2, 10, 1], newdata[2, 10, 1])
        self. assertEqual(
            metadata['voxelsize_mm'][0], newmetadata['voxelsize_mm'][0])
# @TODO there is a bug in SimpleITK. slice voxel size must be same
        # self. assertEqual(metadata['voxelsize_mm'][1],
        # newmetadata['voxelsize_mm'][1])
        self. assertEqual(
            metadata['voxelsize_mm'][2], newmetadata['voxelsize_mm'][2])
        os.remove(filename)

    def test_add_overlay_and_read_one_file_with_overlay(self):
        filename = 'tests_outputs/test_file.dcm'
        filedir = os.path.dirname(filename)

        # number of tested overlay
        i_overlay = 6

        if not os.path.exists('tests_outputs'):
            os.mkdir('tests_outputs')

        data = (np.random.random([30, 100, 120]) * 30).astype(np.int16)
        data[0:5, 20:60, 60:70] += 30
        overlay = np.zeros([512, 512], dtype=np.uint8)
        overlay[450:500, 30:100] = 1

        # metadata = {'voxelsize_mm': [1, 2, 3]}
        dw = dwriter.DataWriter()

        dw.add_overlay_to_slice_file(
            # 'sample_data/jatra_5mm/IM-0001-0019.dcm',
            lisa.dataset.join_sdp('volumetrie/volumetry_slice.DCM'),
            overlay,
            i_overlay,
            filename
        )
        dr = dreader.DataReader()
        newdata, newmetadata = dr.Get3DData('tests_outputs', dataplus_format=False)
        newoverlay = dr.get_overlay()
        # print overlay

        # ed = pyed.sed3(newoverlay[6])
        # ed.show()
        self.assertTrue((newoverlay[i_overlay] == overlay).all())

        # os.remove(filename)
        shutil.rmtree(filedir)

    @attr("slow")
    def test_add_overlay_to_copied_dir(self):
        """
        writes 3d label to copied dicom files
        """
        filedir = 'test_outputs_dir'
        n_files = 3

        # number of tested overlay
        i_overlay = 6

        if not os.path.exists(filedir):
            os.mkdir(filedir)

# open copied data to obtain dcmfilefilelist
        dr = dreader.DataReader()
        data3d, metadata = dr.Get3DData(
            lisa.dataset.sample_data_path() + '/jatra_5mm/', dataplus_format=False
            # 'sample_data/volumetrie/'
        )
# for test we are working only with small number of files (n_files)
        metadata['dcmfilelist'] = metadata['dcmfilelist'][:n_files]

# create overlay
        overlay = np.zeros([n_files, 512, 512], dtype=np.uint8)
        overlay[:, 450:500, 30:100] = 1
# if there is more slides, try more complicated overlay
        overlay[0, 430:460, 20:110] = 1
        overlay[-1:, 470:520, 10:120] = 1

        overlays = {i_overlay: overlay}

        dw = dwriter.DataWriter()
        dw.DataCopyWithOverlay(metadata['dcmfilelist'], filedir, overlays)

# try read written data
        dr = dreader.DataReader()
        newdata, newmetadata = dr.Get3DData(filedir, dataplus_format=False)
        newoverlay = dr.get_overlay()

        self.assertTrue((newoverlay[i_overlay] == overlays[i_overlay]).all())

        # os.remove(filename)
        shutil.rmtree(filedir)
if __name__ == "__main__":
    unittest.main()



#! /usr/bin/python
# -*- coding: utf-8 -*-


import logging
logger = logging.getLogger(__name__)

# import funkcí z jiného adresáře
import sys
import os.path
import copy

path_to_script = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.join(path_to_script, "../extern/pyseg_base/src/"))
sys.path.append(os.path.join(path_to_script, "../extern/sed3/"))
sys.path.append(os.path.join(path_to_script, "../src/"))
import unittest


from PyQt4.QtGui import QFileDialog, QApplication, QMainWindow

import numpy as np

try:
    import pydicom
except ImportError as e:
    import dicom as pydicom
    logger.warning("Used dicom instead of pydicom")
pydicom.config.debug(False)

#
import io3d
import io3d.dcmreaddata as dcmr
import lisa.dataset

class DicomReaderTest(unittest.TestCase):
    interactivetTest = False
#    def setUp(self):
#        #self.dcmdir = os.path.join(path_to_script, '../sample_data/jatra_06mm_jenjatraplus/')
#        self.dcmdir = os.path.join(path_to_script, '../sample_data/jatra_5mm')
#        #self.data3d, self.metadata = dcmr.dcm_read_from_dir(self.dcmdir)
#        reader = dcmr.DicomReader(self.dcmdir)
#        self.data3d = reader.get_3Ddata()
#        self.metadata = reader.get_metaData()

    def test_DicomReader_overlay(self):
        import os.path as op
        # sample_data_path = "~/data/medical/orig/sample_data/"
        # sample_data_path = op.expanduser(sample_data_path)
        # sample_data_path = io3d.datasets.join_path("medical/orig/sample_data/", get_root=True)
        sample_data_path = io3d.datasets.join_path("medical", "orig", get_root=True)
        #import matplotlib.pyplot as plt

        # dcmdir = lisa.dataset.join_sdp('volumetrie/')
        dcmdir = os.path.join(sample_data_path, 'volumetrie')
        # dcmdir = '/home/mjirik/data/medical/data_orig/jatra-kma/jatra_5mm/'
        #self.data3d, self.metadata = dcmr.dcm_read_from_dir(self.dcmdir)
        reader = dcmr.DicomReader(dcmdir)
        overlay = reader.get_overlay()
        #import pdb; pdb.set_trace()
        #plt.imshow(overlay[1][:,:,0])
        #plt.show()

        self.assertEqual(overlay[1][0, 200, 200], 1)
        self.assertEqual(overlay[1][0, 100, 100], 0)

    def test_read_volumetry_overlay_with_dicom_module(self):
        """
        pydicom module is used for load dicom data. Dicom overlay
        is saved on (60xx,3000) bit after bit. Data are decoded and
        each bit is stored as array element.
        """
        # import dicom
        # import sed3
        #import matplotlib.pyplot as plt
        dcmfile = lisa.dataset.join_sdp('volumetrie/volumetry_slice.DCM')
        data = pydicom.read_file(dcmfile)



        # overlay index
        i_overlay = 1
        n_bits = 8


        # On (60xx,3000) are stored ovelays.
        # First is (6000,3000), second (6002,3000), third (6004,3000),
        # and so on.
        dicom_tag1 = 0x6000 + 2*i_overlay

        overlay_raw = data[dicom_tag1, 0x3000].value

        # On (60xx,0010) and (60xx,0011) is stored overlay size
        rows = data[dicom_tag1, 0x0010].value # rows = 512
        cols = data[dicom_tag1, 0x0011].value # cols = 512

        decoded_linear = np.zeros(len(overlay_raw)*n_bits)

        # Decoding data. Each bit is stored as array element
        for i in range(1,len(overlay_raw)):
            for k in range (0, n_bits):
                one_byte = overlay_raw[i]
                if sys.version_info.major == 2:
                    byte_as_int = ord(one_byte)
                else:
                    byte_as_int = one_byte
                decoded_linear[i * n_bits + k] = (byte_as_int >> k) & 0b1

        #overlay = np.array(pol)

        overlay = np.reshape(decoded_linear,[rows,cols])

        #plt.imshow(overlay)
        #plt.show()

        self. assertEqual(overlay[200,200],1)
        self. assertEqual(overlay[100,100],0)
        #pyed = sed3.sed3(overlay)
        #pyed.show()
        #import pdb; pdb.set_trace()







    def test_dcmread(self):

        dcmdir = lisa.dataset.join_sdp('jatra_5mm')
        #dcmdir = '/home/mjirik/data/medical/data_orig/jatra-kma/jatra_5mm/'
        #self.data3d, self.metadata = dcmr.dcm_read_from_dir(self.dcmdir)
        reader = dcmr.DicomReader(dcmdir)
        data3d = reader.get_3Ddata()
        metadata = reader.get_metaData()
#slice size is 512x512
        self.assertEqual(data3d.shape[2],512)
# voxelsize depth = 5 mm
        self.assertEqual(metadata['voxelsize_mm'][0],5)

    def test_dicomread_read(self):
        dcmdir = lisa.dataset.join_sdp('jatra_5mm')
        #dcmdir = '/home/mjirik/data/medical/data_orig/jatra-kma/jatra_5mm/'
        #self.data3d, self.metadata = dcmr.dcm_read_from_dir(self.dcmdir)
        data3d, metadata = io3d.datareader.read(dcmdir, dataplus_format=False)
#slice size is 512x512
        self.assertEqual(data3d.shape[2],512)
# voxelsize depth = 5 mm
        self.assertEqual(metadata['voxelsize_mm'][0],5)

    def test_dcmread_series_number(self):

        dcmdir = lisa.dataset.join_sdp('jatra_5mm')
        #dcmdir = '/home/mjirik/data/medical/data_orig/jatra-kma/jatra_5mm/'
        #self.data3d, self.metadata = dcmr.dcm_read_from_dir(self.dcmdir)
# spravne cislo serie je 7
        reader =  dcmr.DicomReader(dcmdir,series_number = 7)
        data3d = reader.get_3Ddata()
        metadata = reader.get_metaData()
        self.assertEqual(data3d.shape[2],512)
        self.assertEqual(metadata['voxelsize_mm'][0],5)

    @unittest.skipIf(not interactivetTest, 'interactiveTest')
    def test_dcmread_select_series(self):

        #dirpath = dcmr.get_dcmdir_qt()
        dirpath = '/home/mjirik/data/medical/data_orig/46328096/'
        #dirpath = dcmr.get_dcmdir_qt()
        #app = QMainWindow()
        reader = dcmr.DicomReader(dirpath, series_number = 55555)#, #qt_app =app)
        #app.exit()
        self.data3d = reader.get_3Ddata()
        self.metadata = reader.get_metaData()

    #@unittest.skipIf(not interactivetTest, 'interactiveTest')
    @unittest.skip('skip')
    def test_dcmread_get_dcmdir_qt(self):

        dirpath = dcmr.get_dcmdir_qt()
        #self.data3d, self.metadata = dcmr.dcm_read_from_dir(self.dcmdir)
        reader = dcmr.DicomReader(dirpath)
        self.data3d = reader.get_3Ddata()
        self.metadata = reader.get_metaData()

        #sss.visualization()
        #import pdb; pdb.set_trace()


#    def test_synthetic_data_lesions_automatic_localization(self):
#        """
#        Function uses lesions  automatic localization in synthetic data.
#        """
#        #dcmdir = os.path.join(path_to_script,'./../sample_data/matlab/examples/sample_data/DICOM/digest_article/')
## data
#        slab = {'none':0, 'liver':1, 'porta':2, 'lesions':6}
#        voxelsize_mm = np.array([1.0,1.0,1.2])
#
#        segm = np.zeros([256,256,80], dtype=np.int16)
#
#        # liver
#        segm[70:190,40:220,30:60] = slab['liver']
## port
#        segm[120:130,70:220,40:45] = slab['porta']
#        segm[80:130,100:110,40:45] = slab['porta']
#        segm[120:170,130:135,40:44] = slab['porta']
#
#        # vytvoření kopie segmentace - před určením lézí
#        segm_pre = copy.copy(segm)
#
#        segm[150:180,70:105,42:55] = slab['lesions']
#
#
#        data3d = np.zeros(segm.shape)
#        data3d[segm== slab['none']] = 680
#        data3d[segm== slab['liver']] = 1180
#        data3d[segm== slab['porta']] = 1230
#        data3d[segm== slab['lesions']] = 1110
#        #noise = (np.random.rand(segm.shape[0], segm.shape[1], segm.shape[2])*30).astype(np.int16)
#        noise = (np.random.normal(0,30,segm.shape))#.astype(np.int16)
#        data3d = (data3d + noise  ).astype(np.int16)
#
#
#        data={'data3d':data3d,
#                'slab':slab,
#                'voxelsize_mm':voxelsize_mm,
#                'segmentation':segm_pre
#                }
#
## @TODO je tam bug, prohlížeč neumí korektně pracovat s doubly
##        app = QApplication(sys.argv)
##        #pyed = QTSeedEditor(noise )
##        pyed = QTSeedEditor(data3d)
##        pyed.exec_()
##        #img3d = np.zeros([256,256,80], dtype=np.int16)
#
#       # pyed = sed3.sed3(data3d)
#       # pyed.show()
#
#        tumory = lesions.Lesions()
#
#        tumory.import_data(data)
#        tumory.automatic_localization()
#        #tumory.visualization()
#
#
#
## ověření výsledku
#        #pyed = sed3.sed3(outputTmp, contour=segm==slab['porta'])
#        #pyed.show()
#
#        errim = np.abs(
#                (tumory.segmentation == slab['lesions']).astype(np.int) -
#                (segm == slab['lesions']).astype(np.int))
#
## ověření výsledku
#        #pyed = sed3.sed3(errim, contour=segm==slab['porta'])
#        #pyed.show()
##evaluation
#        sum_of_wrong_voxels = np.sum(errim)
#        sum_of_voxels = np.prod(segm.shape)
#
#        #print "wrong ", sum_of_wrong_voxels
#        #print "voxels", sum_of_voxels
#
#        errorrate = sum_of_wrong_voxels/sum_of_voxels
#
#
#        self.assertLess(errorrate,0.1)
#        self.assertLess(errorrate,0.1)
#
#
#

if __name__ == "__main__":
    unittest.main()



#! /usr/bin/python
# -*- coding: utf-8 -*-

import sys
import os.path

path_to_script = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.join(path_to_script, "../../lar-cc/lib/py/"))
import unittest
from nose.plugins.attrib import attr

# import numpy as np


class InterpolationPlasmTest(unittest.TestCase):
    interactiveTests = False
    # interactiveTest = True

    @attr("LAR")
    def test_store_to_SparseMatrix_and_back(self):
        """
        Test has not strong assert part.
        Its passed if any funcion does throw exception.
        """
        from lisa.extern.interpolation_pyplasm import TRIANGLE_DOMAIN
        import pyplasm

        dom2D = TRIANGLE_DOMAIN(5, [[1, 0, 0], [0, 1, 0], [0, 0, 1]])
        self.assertIsInstance(dom2D, list)
        self.assertIsInstance(dom2D[0], pyplasm.xgepy.Hpc)

    # @unittest.skipIf(not interactiveTests, "test is with visualization")
    @attr("LAR")
    def test_complex_sample(self):
        from larcc import INTERVALS, BEZIER, S2, S1, MAP, STRUCT, SKELETON, VIEW
        from lisa.extern.interpolation_pyplasm import TRIANGLE_DOMAIN,\
            TRIANGULAR_COONS_PATCH
        import pyplasm

        BEZIER(S2)
        dom1D = INTERVALS(1)(32) # noqa
        dom2D = TRIANGLE_DOMAIN(32, [[1, 0, 0], [0, 1, 0], [0, 0, 1]])
        Cab0 = BEZIER(S1)([[10, 0, 0], [6, 0, 3], [3, 0, 3], [0, 0, 0]])
        Cbc0 = BEZIER(S1)( # noqa
            [[10, 0, 0], [10, 2, 4], [8, 8, -4], [2, 10, 4], [0, 10, 0]])
        Cbc1 = BEZIER(S2)(
            [[10, 0, 0], [10, 2, 4], [8, 8, -4], [2, 10, 4], [0, 10, 0]])
        Cca0 = BEZIER(S1)([[0, 10, 0], [0, 6, -5], [0, 3, 5], [0, 0, 0]])

        patch = TRIANGULAR_COONS_PATCH([Cab0, Cbc1, Cca0])
        map_input = MAP(patch)(STRUCT(dom2D))
        if self.interactiveTests:
            VIEW(map_input)
            VIEW(SKELETON(1)(map_input))
        self.assertIsInstance(dom2D[0], pyplasm.xgepy.Hpc)
        self.assertIsInstance(map_input, pyplasm.xgepy.Hpc)

if __name__ == "__main__":
    unittest.main()



#! /usr/bin/python
# -*- coding: utf-8 -*-



# import funkcí z jiného adresáře
import sys
import os.path
path_to_script = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.join(path_to_script, "../extern/pyseg_base/src/"))
sys.path.append(os.path.join(path_to_script, "../extern/sed3/"))
#import featurevector
import unittest

import logging
logger = logging.getLogger(__name__)

# import apdb
# apdb.set_trace();
# import scipy.io
import numpy as np

# ----------------- my scripts --------
import argparse

import segmentation
import misc

# Import garbage collector
import gc as garbage

if __name__ == "__main__":

    #logger = logging.getLogger(__name__)
    logger = logging.getLogger()

    logger.setLevel(logging.WARNING)
    ch = logging.StreamHandler()
    logger.addHandler(ch)

    #logger.debug('input params')

    # input parser
    parser = argparse.ArgumentParser(description='Segment vessels from liver')
    parser.add_argument('-dd','--dcmdir',
            default=None,
            help='path to data dir')
    parser.add_argument('-d', '--debug', action='store_true',
            help='run in debug mode')
    parser.add_argument('-bg', '--biggest', action='store_true',
            help='find biggest object')
    parser.add_argument('-ed', '--exampledata', action='store_true',
            help='run unittest')
    parser.add_argument('-i', '--inputfile',  default=None,
            help='input file from organ_segmentation')
    parser.add_argument('-ii', '--defaultinputfile',  action='store_true',
            help='"organ.pkl" as input file from organ_segmentation')
    parser.add_argument('-o', '--outputfile',  default=None,
            help='output file')
    parser.add_argument('-oo', '--defaultoutputfile',  action='store_true',
            help='"vessels.pickle" as output file')
    args = parser.parse_args()


    if args.debug:
        logger.setLevel(logging.DEBUG)

    defaultoutputfile =  "vessels.pkl"
    if args.defaultoutputfile:
        args.outputfile = defaultoutputfile

    if args.exampledata:

        args.dcmdir = '../sample_data/matlab/examples/sample_data/DICOM/digest_article/'

    #else:
    #dcm_read_from_dir('/home/mjirik/data/medical/data_orig/46328096/')
        #data3d, metadata = dcmreaddata.dcm_read_from_dir()
    if args.defaultinputfile:
        args.inputfile = "organ.pkl"

    if args.inputfile == None:
        oseg = organ_segmentation.OrganSegmentation(args.dcmdir, working_voxelsize_mm = 6, autocrop = True, autocrop_margin_mm = [10,10,10])
        oseg.interactivity()
        # Uvolneni pameti
        garbage.collect()
        print( "Volume " + str(oseg.get_segmented_volume_size_mm3()/1000000.0) + ' [l]' )
# get data in list
        data = oseg.export()
    else:
        data = misc.obj_from_file(args.inputfile, filetype = 'pickle')

    import sed3
    # pyed = sed3.sed3(oseg.orig_scale_segmentation)
    #pyed.show()
# information about crop
    #cri = oseg.crinfo
    #oseg.data3d = oseg.data3d[cri[0][0]:cri[0][1],cri[1][0]:cri[1][1],cri[2][0]:cri[2][1]]
    #pyed = sed3.sed3(oseg.data3d, contour = oseg.orig_scale_segmentation)

    print('slab', data['slab'])
    #import ipdb; ipdb.set_trace()  # BREAKPOINT
    #pyed = sed3.sed3(data['data3d'], contour = data['segmentation'])
    #pyed.show()
    #import pdb; pdb.set_trace()

    outputTmp = segmentation.vesselSegmentation(
        data['data3d'],
        segmentation = data['segmentation'],
        #segmentation = oseg.orig_scale_segmentation,
        threshold = -1,
        inputSigma = 0.15,
        dilationIterations = 2,
        nObj = 1,
        biggestObjects = args.biggest,
#        dataFiltering = True,
        interactivity = True,
        binaryClosingIterations = 2,
        binaryOpeningIterations = 0)

    # Uvolneni pameti
    garbage.collect()

    import inspector
    inspect = inspector.inspector(outputTmp)
    output = inspect.run()

    # Uvolneni pameti
    del(inspect)
    garbage.collect()

    #pyed = sed3.sed3(outputTmp)
    #pyed.show()
# segmentation labeling
    #slab={}
    data['slab']['none'] = 0
    data['slab']['liver'] = 1
    data['slab']['porta'] = 2


    #print np.max(output)
    #import pdb; pdb.set_trace()
    #data = {}
    #data['data3d'] = oseg.data3d
    #data['crinfo'] = oseg.crinfo
    #data['segmentation'] = oseg.segmentation
    data['segmentation'][output] = data['slab']['porta']
    #data['slab'] = slab


    pyed = sed3.sed3(data['data3d'],  contour=data['segmentation']==data['slab']['porta'])
    pyed.show()

    #pyed = sed3.sed3(data['segmentation'])
    #pyed.show()
    # Uvolneni pameti
    garbage.collect()


    if args.outputfile == None:

        savestring = raw_input ('Save output data? (y/n): ')
        #sn = int(snstring)
        if savestring in ['Y','y']:
            pth, filename = os.path.split(os.path.normpath(args.inputfile))

            misc.obj_to_file(data,
                             defaultoutputfile + '-' + filename,
                             filetype = 'pickle')
    else:
        misc.obj_to_file(data, args.outputfile, filetype = 'pickle')




#!/usr/bin/env python
# -*- coding: utf-8 -*-
""" Processing of exceptions with lisa. """

import logging
logger = logging.getLogger(__name__)


import traceback
# Import smtplib for the actual sending function
import smtplib

# Import the email modules we'll need
from email.mime.text import MIMEText


def reportException(exception):
    excstr = traceback.format_exc()
    try:
        sendMail(excstr, 'Lisa exception: ' + str(exception))
    except Exception as e:
        logger.debug("Problems with sending exception report")
        logger.debug(traceback.format_exc())
        # logger.debug(str(e))
        # logger.debug("Original exception:")
        # logger.debug(str(exception))
        # logger.debug(excstr)

    logger.exception(excstr)

    raise(exception)


def sendMail(mailcontent, subject='None'):
    me = 'mjirik@kky.zcu.cz'
    you = 'miroslav.jirik@gmail.com'

# Open a plain text file for reading.  For this example, assume that
# the text file contains only ASCII characters.
# fp = open(textfile, 'rb')
# # Create a text/plain message
    msg = MIMEText(mailcontent)

# me == the sender's email address
# you == the recipient's email address
    msg['Subject'] = subject
    msg['From'] = me
    msg['To'] = you

# Send the message via our own SMTP server, but don't include the
# envelope header.
    s = smtplib.SMTP('localhost')
    s.sendmail(me, [you], msg.as_string())
    s.quit()
    logger.debug('Subject: ', subject)
    logger.warn('Mail content')
    logger.debug(mailcontent)



#! /usr/bin/python
# -*- coding: utf-8 -*-


# import funkcí z jiného adresáře
import os.path

import sys
path_to_script = os.path.dirname(os.path.abspath(__file__))
# sys.path.append(os.path.join(path_to_script, "../extern/pyseg_base/src/"))
# sys.path.append(os.path.join(path_to_script, "../extern/sed3/"))
# sys.path.append(os.path.join(path_to_script, "../src/"))
import unittest

from nose.plugins.attrib import attr
import numpy as np

from PyQt4.QtGui import QApplication
import sed3

from lisa import organ_segmentation
from imtools import segmentation


class VesselsSegmentationTest(unittest.TestCase):
    interactiveTest = False

    # @unittest.skip("demonstrating skipping")
    # @unittest.skipIf(not interactiveTest, "interactive test")
    @attr('interactive')
    def test_whole_organ_segmentation_interactive(self):
        pass

    def synthetic_data(self):
        # data
        slab = {'none': 0, 'liver': 1, 'porta': 2, "hepatic_veins": 3}
        voxelsize_mm = np.array([1.0, 1.0, 1.2])

        segm = np.zeros([256, 256, 80], dtype=np.int16)

        # liver
        segm[70:180, 40:190, 30:60] = slab['liver']
        # porta
        segm[120:130, 70:190, 40:45] = slab['porta']
        segm[80:130, 100:110, 40:45] = slab['porta']
        segm[120:170, 130:135, 40:44] = slab['porta']
        segm[150:160, 70:190, 51:55] = slab['hepatic_veins']

        data3d = np.zeros(segm.shape)
        data3d[segm == slab['liver']] = 156
        data3d[segm == slab['porta']] = 206
        noise = (np.random.normal(0, 30, segm.shape))  # .astype(np.int16)
        data3d = (data3d + noise).astype(np.int16)
        return data3d, segm, voxelsize_mm, slab

    def test_synthetic_data_segmentation(self):
        """
        Function uses organ_segmentation for synthetic box object
        segmentation.
        """

        data3d, segm, voxelsize_mm, slab = self.synthetic_data()
# @TODO je tam bug, prohlížeč neumí korektně pracovat s doubly
#        app = QApplication(sys.argv)
#        #pyed = QTSeedEditor(noise )
#        pyed = QTSeedEditor(data3d)
#        pyed.exec_()
#        #img3d = np.zeros([256,256,80], dtype=np.int16)

        # pyed = sed3.sed3(data3d)
        # pyed.show()

        outputTmp = segmentation.vesselSegmentation(
            data3d,  # .astype(np.uint8),
            segmentation=(segm == slab['liver']),  # .astype(np.uint8),
            # segmentation = oseg.orig_scale_segmentation,
            voxelsize_mm=voxelsize_mm,
            threshold=180,
            inputSigma=0.15,
            aoi_dilation_iterations=2,
            nObj=1,
            interactivity=False,
            biggestObjects=True,
            binaryClosingIterations=5,
            binaryOpeningIterations=1)

# ověření výsledku
        # pyed = sed3.sed3(outputTmp, contour=segm==slab['porta'])
        # pyed.show()

# @TODO opravit chybu v vesselSegmentation
        outputTmp = (outputTmp == 2)
        errim = np.abs(
            outputTmp.astype(np.int) - (segm == slab['porta']).astype(np.int)
        )

# ověření výsledku
        # pyed = sed3.sed3(errim, contour=segm==slab['porta'])
        # pyed.show()
# evaluation
        sum_of_wrong_voxels = np.sum(errim)
        sum_of_voxels = np.prod(segm.shape)

        # print "wrong ", sum_of_wrong_voxels
        # print "voxels", sum_of_voxels

        errorrate = sum_of_wrong_voxels/sum_of_voxels

        # import pdb; pdb.set_trace()

        self.assertLess(errorrate, 0.1)

    def test_virtual_resection(self):
        """
        Make virtual resection on synthetic data
        """
        import lisa.virtual_resection as vr
        data3d, segm, voxelsize_mm, slab = self.synthetic_data()
        seeds = np.zeros([256, 256, 80], dtype=np.int16)
        seeds[125, 160, 44] = 1
        datap = {'data3d': data3d, 'segmentation': segm,
                 'voxelsize_mm': voxelsize_mm, 'slab': slab}

        datap = vr.resection(
            datap, method='PV',
            interactivity=False, seeds=seeds
        )

        resected = np.sum(
            datap['segmentation'] == datap['slab']['resected_liver'])
        remaining = np.sum(
            datap['segmentation'] == datap['slab']['liver'])
        ratio = np.double(resected)/(resected+remaining)

        self.assertGreater(ratio, 0.17)
        self.assertLess(ratio, 0.19)

    def test_split_by_two_vessels(self):
        """
        Make virtual resection on synthetic data
        """
        import lisa.virtual_resection as vr
        data3d, segm, voxelsize_mm, slab = self.synthetic_data()
        seeds = np.zeros([256, 256, 80], dtype=np.int16)
        seeds[125, 160, 44] = 1
        datap = {'data3d': data3d, 'segmentation': segm,
                 'voxelsize_mm': voxelsize_mm, 'slab': slab}

        oseg = organ_segmentation.OrganSegmentation()
        oseg.import_dataplus(datap)

        oseg.split_organ_by_two_vessels(
            seed_label1="porta",
            seed_label2="hepatic_veins",
            organ_label="liver",
            output_label1="liver",
            output_label2="liver_remnant"
        )

        resected = np.sum(
            datap['segmentation'] == datap['slab']['liver'])
        remaining = np.sum(
            datap['segmentation'] == datap['slab']['liver_remnant'])
        ratio = np.double(resected)/(resected+remaining)
        # ed = sed3.sed3(datap["segmentation"])
        # ed.show()
        # print(ratio)

        self.assertGreater(ratio, 0.6)
        self.assertLess(ratio, 0.8)

    @attr('slow')
    def test_real_data_segmentation(self):
        import lisa.dataset
        app = QApplication(sys.argv)

        dcmdir = os.path.join(lisa.dataset.sample_data_path(), 'jatra_5mm')

        oseg = organ_segmentation.OrganSegmentation(dcmdir,
                                                    working_voxelsize_mm=4)
        oseg.add_seeds_mm([81], [120], [120],
                          label=1, radius=30)
        oseg.add_seeds_mm([81], [170, 220, 250], [250, 250, 200],
                          label=2, radius=30)
        oseg.add_seeds_mm([100], [170], [140],
                          label=10, radius=5)
        # oseg.interactivity(min_val=-200, max_val=200)
        oseg.ninteractivity()

        ed = sed3.sed3(oseg.data3d, contour=oseg.segmentation, seeds=oseg.seeds)
        ed.show()

        oseg.portalVeinSegmentation(seeds=(oseg.seeds==10))

#
#
#        outputTmp = segmentation.vesselSegmentation(
#            data3d,
#            segmentation = segm==slab['liver'],
#            #segmentation = oseg.orig_scale_segmentation,
#            voxelsize_mm = voxelsize_mm,
#            threshold = 1204,
#            inputSigma = 0.15,
#            dilationIterations = 2,
#            nObj = 1,
#            interactivity = False,
#            binaryClosingIterations = 5,
#            binaryOpeningIterations = 1)
#
#

    @attr('interactive')
    def test_ui_threshold_same_params_as_portal_segmentation(self):

        params = {
            'threshold': -1,
            'inputSigma': 0.15,
            'dilationIterations': 10,
            'nObj': 1,
            'biggestObjects': False,
            'useSeedsOfCompactObjects': True,
            'interactivity': True,
            'binaryClosingIterations': 2,
            'binaryOpeningIterations': 0
        }
        import imtools
        import imtools.sample_data
        datap = imtools.sample_data.generate()
        app = QApplication(sys.argv)
        outputTmp = segmentation.vesselSegmentation(
            datap['data3d'],
            datap['segmentation'],
            # datap['voxelsize_mm'],
            **params)
        # uit.run()
        # plt.show()
if __name__ == "__main__":
    unittest.main()



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim:fenc=utf-8
#
# Copyright © %YEAR%  <>
#
# Distributed under terms of the %LICENSE% license.

"""

"""

import logging

logger = logging.getLogger(__name__)
import argparse
from PyQt4.QtGui import QGridLayout, QLabel, QPushButton, QLineEdit
from PyQt4 import QtGui
import sys
import copy

class DictEdit(QtGui.QWidget):
    def __init__(self, dictionary):
        super(DictEdit, self).__init__()

        self.dictionary = dictionary
        self.autoValue = 1
        self.initUI()


    def initUI(self):
        self.mainLayout = QGridLayout(self)
        self.mainLayout.addWidget(QLabel("Key"), 0, 1)
        self.mainLayout.addWidget(QLabel("Value"), 0, 2)

        self.slabKeys = []
        self.slabValues = []
        self.pos = 1
        self.ui_label_lines = []

        self.initLines()

        self.lblSlabError = QLabel()
        self.lblSlabError.setStyleSheet("color: red;");
        self.mainLayout.addWidget(self.lblSlabError, 20, 1, 1, 2)

        btnAddLabel = QPushButton("+", self)
        btnAddLabel.setFixedWidth(30)
        btnAddLabel.setStyleSheet('QPushButton {background-color: green; color: #FFFFFF}')
        btnAddLabel.clicked.connect(self.addLabel)
        self.mainLayout.addWidget(btnAddLabel, 21, 3)

        btnSaveSlab = QPushButton("Save", self)
        btnSaveSlab.clicked.connect(self.saveSlab)
        self.mainLayout.addWidget(btnSaveSlab, 21, 1)
        self.btnSaveSlab = btnSaveSlab

        self.btnDiscard = QPushButton("Discard", self)
        self.btnDiscard.clicked.connect(self.discardChanges)
        self.mainLayout.addWidget(self.btnDiscard, 21, 2)

    def initLines(self):
        for key, value in self.dictionary.slab.items(): #self.oseg.slab.items():
            if key == "none":
                continue
            keyW = QLineEdit(key)
            valueW = QLineEdit(str(value))

            btnDlt = QPushButton(u"\u00D7")
            btnDlt.setFixedWidth(30)
            btnDlt.setStyleSheet('QPushButton {background-color: red; color: #FFFFFF}')

            ui_label_line = [self.pos, keyW, valueW, btnDlt]
            self.ui_label_lines.append(ui_label_line)
            btnDlt.clicked.connect(lambda state, x=ui_label_line: self.deleteLine(x))

            self.mainLayout.addWidget(keyW, self.pos, 1)
            self.mainLayout.addWidget(valueW, self.pos, 2)
            self.mainLayout.addWidget(btnDlt, self.pos, 3)


            self.slabKeys.append(keyW)
            self.slabValues.append(valueW)
            self.pos += 1

    def deleteLines(self):
        for i in range(0, len(self.ui_label_lines)):
            self.ui_label_lines[i][1].deleteLater()
            self.ui_label_lines[i][2].deleteLater()
            self.ui_label_lines[i][3].deleteLater()

        self.slabKeys = []
        self.slabValues = []
        self.pos = 1
        self.ui_label_lines = []

    def deleteLine(self, event):
        self.slabKeys.remove(event[1])
        self.slabValues.remove(event[2])

        event[1].deleteLater()
        event[2].deleteLater()
        event[3].deleteLater()

        self.sortLines(event[0])
        self.ui_label_lines.remove(event)
        self.pos -= 1

    def sortLines(self, whence):
        for i in range(whence, len(self.ui_label_lines)):
            self.ui_label_lines[i][0] = i
            self.mainLayout.addWidget(self.ui_label_lines[i][1], i, 1)
            self.mainLayout.addWidget(self.ui_label_lines[i][2], i, 2)
            self.mainLayout.addWidget(self.ui_label_lines[i][3], i, 3)

    def addLabel(self):
        if self.pos < 12:
            actualValues = []
            for i in range(0, len(self.slabValues)):
                actualValues.append(int(self.slabValues[i].text()))
            while self.autoValue in actualValues:
                self.autoValue += 1

            keyW = QLineEdit()
            valueW = QLineEdit(str(self.autoValue))
            btnDlt = QPushButton(u"\u00D7")
            btnDlt.setFixedWidth(30)
            btnDlt.setStyleSheet('QPushButton {background-color: red; color: #FFFFFF}')
            ui_label_line = [self.pos, keyW, valueW, btnDlt]
            self.ui_label_lines.append(ui_label_line)
            btnDlt.clicked.connect(lambda state, x=ui_label_line: self.deleteLine(x))

            self.mainLayout.addWidget(keyW, self.pos, 1)
            self.mainLayout.addWidget(valueW, self.pos, 2)
            self.mainLayout.addWidget(btnDlt, self.pos, 3)
            self.slabKeys.append(keyW)
            self.slabValues.append(valueW)

            self.pos += 1
        else:
            self.lblSlabError.setText("You cannot add new label")

    def saveSlab(self):
        self.lblSlabError.setText('')
        newSlab = {}
        for i in range(0, len(self.slabKeys)):
            wk = str(self.slabKeys[i].text())
            wv = str(self.slabValues[i].text())
            if wk != '':
                newSlab[wk] = wv
            elif wv != '':
                self.lblSlabError.setText("You have to name key!")
        self.dictionary.slab = newSlab

    def discardChanges(self):
        self.deleteLines()
        self.initLines()

    def getDict(self):
        return self.dictionary




def main():
    logger = logging.getLogger()

    logger.setLevel(logging.DEBUG)
    ch = logging.StreamHandler()
    logger.addHandler(ch)

    # create file handler which logs even debug messages
    # fh = logging.FileHandler('log.txt')
    # fh.setLevel(logging.DEBUG)
    # formatter = logging.Formatter(
    #     '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    # fh.setFormatter(formatter)
    # logger.addHandler(fh)
    # logger.debug('start')

    # input parser
    parser = argparse.ArgumentParser(
        description=__doc__
    )
    # parser.add_argument(
    #     '-i', '--inputfile',
    #     default=None,
    #     required=True,
    #     help='input file'
    # )
    parser.add_argument(
        '--dict',
        default="{'jatra':2, 'ledviny':7}",
        # required=True,
        help='input dict'
    )
    parser.add_argument(
        '-d', '--debug', action='store_true',
        help='Debug mode')
    args = parser.parse_args()

    if args.debug:
        ch.setLevel(logging.DEBUG)

    app = QtGui.QApplication(sys.argv)

    # w = QtGui.QWidget()
    # w = DictEdit(dictionary={'jatra':2, 'ledviny':7})
    w = DictEdit(dictionary=eval(args.dict))
    w.resize(250, 150)
    w.move(300, 300)
    w.setWindowTitle('Simple')
    w.show()

    sys.exit(app.exec_())


if __name__ == "__main__":
    main()


__author__ = 'Ryba'
import numpy as np
import matplotlib.pyplot as plt
import skimage.exposure as skexp
from skimage.segmentation import mark_boundaries
import os
import glob
import pydicom
# import cv2
# from skimage import measure
import skimage.measure as skimea
import skimage.morphology as skimor
import skimage.filters as skifil
import scipy.stats as scista

#----------------------------------------------------------------------------------------------------------------------
#----------------------------------------------------------------------------------------------------------------------
def get_seeds(im, minT=0.95, maxT=1.05, minInt=0, maxInt=255, debug=False):
    vals = im[np.where(np.logical_and(im>=minInt, im<=maxInt))]
    hist, bins = skexp.histogram(vals)
    max_peakIdx = hist.argmax()

    minT *= bins[max_peakIdx]
    maxT *= bins[max_peakIdx]
    histTIdxs = (bins >= minT) * (bins <= maxT)
    histTIdxs = np.nonzero(histTIdxs)[0]
    class1TMin = minT
    class1TMax = maxT

    seed_mask = np.where( (im >= class1TMin) * (im <= class1TMax), 1, 0)

    if debug:
        plt.figure()
        plt.plot(bins, hist)
        plt.hold(True)

        plt.plot(bins[max_peakIdx], hist[max_peakIdx], 'ro')
        plt.plot(bins[histTIdxs], hist[histTIdxs], 'r')
        plt.plot(bins[histTIdxs[0]], hist[histTIdxs[0]], 'rx')
        plt.plot(bins[histTIdxs[-1]], hist[histTIdxs[-1]], 'rx')
        plt.title('Image histogram and its class1 = maximal peak (red dot) +/- minT/maxT % of its density (red lines).')
        plt.show()

    #minT *= hist[max_peakIdx]
    #maxT *= hist[max_peakIdx]
    #histTIdxs = (hist >= minT) * (hist <= maxT)
    #histTIdxs = np.nonzero(histTIdxs)[0]
    #histTIdxs = histTIdxs.astype(np.int)minT *= hist[max_peakIdx]
    #class1TMin = bins[histTIdxs[0]]
    #class1TMax = bins[histTIdxs[-1]

    #if debug:
    #    plt.figure()
    #    plt.plot(bins, hist)
    #    plt.hold(True)
    #
    #    plt.plot(bins[max_peakIdx], hist[max_peakIdx], 'ro')
    #    plt.plot(bins[histTIdxs], hist[histTIdxs], 'r')
    #    plt.plot(bins[histTIdxs[0]], hist[histTIdxs[0]], 'rx')
    #    plt.plot(bins[histTIdxs[-1]], hist[histTIdxs[-1]], 'rx')
    #    plt.title('Image histogram and its class1 = maximal peak (red dot) +/- minT/maxT % of its density (red lines).')
    #    plt.show()

    return seed_mask, class1TMin, class1TMax


#----------------------------------------------------------------------------------------------------------------------
#----------------------------------------------------------------------------------------------------------------------
def seeds2superpixels(seed_mask, superpixels, debug=False, im=None):
    seeds = np.argwhere(seed_mask)
    superseeds = np.zeros_like(seed_mask)

    for s in seeds:
        label = superpixels[s[0], s[1]]
        superseeds = np.where(superpixels==label, 1, superseeds)

    if debug:
        plt.figure(), plt.gray()
        plt.subplot(121), plt.imshow(im), plt.hold(True), plt.plot(seeds[:,1], seeds[:,0], 'ro'), plt.axis('image')
        plt.subplot(122), plt.imshow(im), plt.hold(True), plt.plot(seeds[:,1], seeds[:,0], 'ro'),
        plt.imshow(mark_boundaries(im, superseeds, color=(1,0,0))), plt.axis('image')
        plt.show()

    return superseeds


#----------------------------------------------------------------------------------------------------------------------
#----------------------------------------------------------------------------------------------------------------------
def intensity_range2superpixels(im, superpixels, intMinT=0.95, intMaxT=1.05, debug=False, intMin=0, intMax=255):#, fromInt=0, toInt=255):

    superseeds = np.zeros_like(superpixels)

    #if not intMin and not intMax:
    #    hist, bins = skexp.histogram(im)
    #
    #    #zeroing values that are lower/higher than fromInt/toInt
    #    toLow = np.where(bins < fromInt)
    #    hist[toLow] = 0
    #    toHigh = np.where(bins > toInt)
    #    hist[toHigh] = 0
    #
    #    max_peakIdx = hist.argmax()
    #    intMin = intMinT * bins[max_peakIdx]
    #    intMax = intMaxT * bins[max_peakIdx]

    sp_means = np.zeros(superpixels.max()+1)
    for sp in range(superpixels.max()+1):
        values = im[np.where(superpixels==sp)]
        mean = np.mean(values)
        sp_means[sp] = mean

    idxs = np.argwhere(np.logical_and(sp_means>=intMin, sp_means<=intMax))
    for i in idxs:
        superseeds = np.where(superpixels==i[0], 1, superseeds)

    if debug:
        plt.figure(), plt.gray()
        plt.imshow(im), plt.hold(True), plt.imshow(mark_boundaries(im, superseeds, color=(1,0,0)))
        plt.axis('image')
        plt.show()

    return superseeds


def show_slice(data, segmentation=None, lesions=None, show='True'):
    plt.figure()
    plt.gray()
    plt.imshow(data)

    if segmentation is not None:
        plt.hold(True)
        contours = skimea.find_contours(segmentation, 1)
        for contour in contours:
            plt.plot(contour[:, 1], contour[:, 0], 'b', linewidth=2)

    if lesions is not None:
        plt.hold(True)
        contours = skimea.find_contours(lesions, 1)
        for contour in contours:
            plt.plot(contour[:, 1], contour[:, 0], 'r', linewidth=2)

    plt.axis('image')

    if show:
        plt.show()


def change_slice_index(data):
    nSlices = data.shape[2]
    data_reshaped = np.zeros(np.hstack((data.shape[2],data.shape[0],data.shape[1])))
    for i in range(nSlices):
        data_reshaped[i,:,:] = data[:,:,i]
    return data_reshaped


def read_data(dcmdir, indices=None, wildcard='*.dcm', type=np.int16):

    dcmlist = []
    for infile in glob.glob(os.path.join(dcmdir, wildcard)):
        dcmlist.append(infile)

    if indices == None:
        indices = range(len(dcmlist))

    data3d = []
    for i in range(len(indices)):
        ind = indices[i]
        onefile = dcmlist[ind]
        if wildcard == '*.dcm':
            data = pydicom.read_file(onefile)
            data2d = data.pixel_array
            try:
                data2d = (np.float(data.RescaleSlope) * data2d) + np.float(data.RescaleIntercept)
            except:
                print('problem with RescaleSlope and RescaleIntercept')
        else:
            data2d = cv2.imread(onefile, 0)

        if len(data3d) == 0:
            shp2 = data2d.shape
            data3d = np.zeros([shp2[0], shp2[1], len(indices)], dtype=type)

        data3d[:,:,i] = data2d

    #need to reshape data to have slice index (ndim==3)
    if data3d.ndim == 2:
        data3d.resize(np.hstack((data3d.shape,1)))

    return data3d


def windowing(data, level=50, width=300, sub1024=False, sliceId=2):
    #srovnani na standardni skalu = odecteni 1024HU
    if sub1024:
        data -= 1024

    #zjisteni minimalni a maximalni density
    minHU = level - width
    maxHU = level + width

    if data.ndim == 3:
        if sliceId == 2:
            for idx in range(data.shape[2]):
                #rescalovani intenzity tak, aby skala <minHU, maxHU> odpovidala intervalu <0,255>
                data[:,:,idx] = skexp.rescale_intensity(data[:,:,idx], in_range=(minHU, maxHU), out_range=(0, 255))
        elif sliceId == 0:
            for idx in range(data.shape[0]):
                #rescalovani intenzity tak, aby skala <minHU, maxHU> odpovidala intervalu <0,255>
                data[idx,:,:] = skexp.rescale_intensity(data[idx,:,:], in_range=(minHU, maxHU), out_range=(0, 255))
    else:
        data = skexp.rescale_intensity(data, in_range=(minHU, maxHU), out_range=(0, 255))

    return data.astype(np.uint8)


def smoothing(data, d=10, sigmaColor=10, sigmaSpace=10, sliceId=2):
    if data.ndim == 3:
        if sliceId == 2:
            for idx in range(data.shape[2]):
                data[:,:,idx] = cv2.bilateralFilter( data[:,:,idx], d=d, sigmaColor=sigmaColor, sigmaSpace=sigmaSpace )
        elif sliceId == 0:
            for idx in range(data.shape[0]):
                data[idx,:,:] = cv2.bilateralFilter( data[idx,:,:], d=d, sigmaColor=sigmaColor, sigmaSpace=sigmaSpace )
    else:
        data = cv2.bilateralFilter( data, d=d, sigmaColor=sigmaColor, sigmaSpace=sigmaSpace )
    return data

def smoothing_bilateral(data, sigma_space=15, sigma_color=0.05, pseudo_3D='True', sliceId=2):
    if data.ndim == 3 and pseudo_3D:
        if sliceId == 2:
            for idx in range(data.shape[2]):
                temp = skifil.denoise_bilateral(data[:, :, idx], sigma_range=sigma_color, sigma_spatial=sigma_space)
                data[idx, :, :] = (255 * temp).astype(np.uint8)
        elif sliceId == 0:
            for idx in range(data.shape[0]):
                temp = skifil.denoise_bilateral(data[idx, :, :], sigma_range=sigma_color, sigma_spatial=sigma_space)
                data[idx, :, :] = (255 * temp).astype(np.uint8)
    else:
        data = skifil.denoise_bilateral(data, sigma_range=sigma_color, sigma_spatial=sigma_space)
        data = (255 * data).astype(np.uint8)
    return data

def smoothing_tv(data, weight, pseudo_3D='True', multichannel=False, sliceId=2):
    if data.ndim == 3 and pseudo_3D:
        if sliceId == 2:
            for idx in range(data.shape[2]):
                temp = skifil.denoise_tv_chambolle(data[:, :, idx], weight=weight, multichannel=multichannel)
                data[:, :, idx] = (255 * temp).astype(np.uint8)
        elif sliceId == 0:
            for idx in range(data.shape[0]):
                temp = skifil.denoise_tv_chambolle(data[idx, :, :], weight=weight, multichannel=multichannel)
                data[idx, :, :] = (255 * temp).astype(np.uint8)
    else:
        data = skifil.denoise_tv_chambolle(data, weight=weight, multichannel=False)
        data = (255 * data).astype(np.uint8)
    return data

def canny(data, sigma=1, sliceId=2):
    edges = np.zeros(data.shape, dtype=np.bool)
    if sliceId == 2:
        for idx in range(data.shape[2]):
            edges[:, :, idx] = skifil.canny(data[:, :, idx], sigma=sigma)
    elif sliceId == 0:
        for idx in range(data.shape[0]):
            edges[idx, :, :] = skifil.canny(data[idx, :, :], sigma=sigma)
    return edges

def scharr(data, sliceId=2):
    edges = np.zeros(data.shape)
    if sliceId == 2:
        for idx in range(data.shape[2]):
            edges[:, :, idx] = skifil.scharr(data[:, :, idx])
    elif sliceId == 0:
        for idx in range(data.shape[0]):
            edges[idx, :, :] = skifil.scharr(data[idx, :, :])
    return edges

def sobel(data, sliceId=2):
    edges = np.zeros(data.shape)
    if sliceId == 2:
        for idx in range(data.shape[2]):
            edges[:, :, idx] = skifil.sobel(data[:, :, idx])
    elif sliceId == 0:
        for idx in range(data.shape[0]):
            edges[idx, :, :] = skifil.sobel(data[idx, :, :])
    return edges

def roberts(data, sliceId=2):
    edges = np.zeros(data.shape)
    if sliceId == 2:
        for idx in range(data.shape[2]):
            edges[:, :, idx] = skifil.roberts(data[:, :, idx])
    elif sliceId == 0:
        for idx in range(data.shape[0]):
            edges[idx, :, :] = skifil.roberts(data[idx, :, :])
    return edges

def analyse_histogram(data, roi=None, debug=False, dens_min=20, dens_max=255, minT=0.95, maxT=1.05):
    if roi == None:
        #roi = np.ones(data.shape, dtype=np.bool)
        roi = np.logical_and(data >= dens_min, data <= dens_max)

    smooth = smoothing_tv(data, weight=0.1, sliceId=0)
    voxels = data[np.nonzero(roi)]
    hist, bins = skexp.histogram(voxels)
    max_peakIdx = hist.argmax()

    minT = minT * hist[max_peakIdx]
    maxT = maxT * hist[max_peakIdx]
    histTIdxs = (hist >= minT) * (hist <= maxT)
    histTIdxs = np.nonzero(histTIdxs)[0]
    histTIdxs = histTIdxs.astype(np.int)

    class1TMin = bins[histTIdxs[0]]
    class1TMax = bins[histTIdxs[-1]]

    # liver = data * (roi > 0)
    liver = smooth * (roi > 0)
    class1 = np.where( (liver >= class1TMin) * (liver <= class1TMax), 1, 0)

    if debug:
        plt.figure()
        plt.plot(bins, hist)
        plt.hold(True)

        plt.plot(bins[max_peakIdx], hist[max_peakIdx], 'ro')
        plt.plot(bins[histTIdxs], hist[histTIdxs], 'r')
        plt.plot(bins[histTIdxs[0]], hist[histTIdxs[0]], 'rx')
        plt.plot(bins[histTIdxs[-1]], hist[histTIdxs[-1]], 'rx')
        plt.title('Histogram of liver density and its class1 = maximal peak (red dot) +-5% of its density (red line).')
        plt.show()

    return class1


def intensity_probability(data, std=20, roi=None, dens_min=5, dens_max=255):
    if roi == None:
        # roi = np.logical_and(data >= dens_min, data <= dens_max)
        roi = np.ones(data.shape, dtype=np.bool)
    voxels = data[np.nonzero(roi)]
    hist, bins = skexp.histogram(voxels)

    # zeroing histogram outside interval <dens_min, dens_max>
    # update: it's not necessary if there's a roi provided
    hist[:dens_min] = 0
    hist[dens_max:] = 0

    max_id = hist.argmax()
    mu = round(bins[max_id])

    prb = scista.norm(loc=mu, scale=std)
    probs_L = prb.pdf(voxels)

    print('liver pdf: mu = %i, std = %i'%(mu, std))

    # plt.figure()
    # plt.plot(bins, hist)
    # plt.hold(True)
    # plt.plot(mu, hist[max_id], 'ro')
    # plt.show()

    probs = np.zeros(data.shape)

    coords = np.argwhere(roi)
    n_elems = coords.shape[0]
    for i in range(n_elems):
        if data.ndim == 3:
            probs[coords[i,0], coords[i,1], coords[i,2]] = probs_L[i]
        else:
            probs[coords[i,0], coords[i,1]] = probs_L[i]

    return probs, mu


def get_zunics_compatness(obj):
    m000 = obj.sum()
    m200 = get_central_moment(obj, 2, 0, 0)
    m020 = get_central_moment(obj, 0, 2, 0)
    m002 = get_central_moment(obj, 0, 0, 2)
    term1 = (3**(5./3)) / (5 * (4*np.pi)**(2./3))
    term2 = m000**(5./3) / (m200 + m020 + m002)
    K = term1 * term2
    return K


def get_central_moment(obj, p, q, r):
    elems = np.argwhere(obj)
    m000 = obj.sum()
    m100 = (elems[:,0]).sum()
    m010 = (elems[:,1]).sum()
    m001 = (elems[:,2]).sum()
    xc = m100 / m000
    yc = m010 / m000
    zc = m001 / m000

    mom = 0
    for el in elems:
        mom += (el[0] - xc)**p + (el[1] - yc)**q + (el[2] - zc)**r

    return mom


def opening3D(data, selem=skimor.disk(3)):
    for i in range(data.shape[0]):
        data[i,:,:] = skimor.binary_opening(data[i,:,:], selem)
    return data


def closing3D(data, selem=skimor.disk(3)):
    for i in range(data.shape[0]):
        data[i,:,:] = skimor.binary_closing(data[i,:,:], selem)
    return data


def resize3D(data, scale, sliceId=2):
    if sliceId == 2:
        n_slices = data.shape[2]
        new_shape = cv2.resize(data[:,:,0], None, fx=scale, fy=scale).shape
        new_data = np.zeros(np.hstack((new_shape,n_slices)))
        for i in range(n_slices):
            new_data[:,:,i] = cv2.resize(data[:,:,i], None, fx=scale, fy=scale)
    elif sliceId == 0:
        n_slices = data.shape[0]
        new_shape = cv2.resize(data[0,:,:], None, fx=scale, fy=scale).shape
        new_data = np.zeros(np.hstack((n_slices, np.array(new_shape))))
        for i in range(n_slices):
            new_data[i,:,:] = cv2.resize(data[i,:,:], None, fx=scale, fy=scale)
    return new_data


def get_overlay(mask, alpha=0.3, color='r'):
    layer = None
    if color == 'r':
        layer = np.dstack((255*mask, np.zeros_like(mask), np.zeros_like(mask), alpha * mask))
    elif color == 'g':
        layer = alpha * np.dstack((np.zeros_like(mask), mask, np.zeros_like(mask)))
    elif color == 'b':
        layer = alpha * np.dstack((np.zeros_like(mask), np.zeros_like(mask), mask))
    elif color == 'c':
        layer = alpha * np.dstack((np.zeros_like(mask), mask, mask))
    elif color == 'm':
        layer = alpha * np.dstack((mask, np.zeros_like(mask), mask))
    elif color == 'y':
        layer = alpha * np.dstack((mask, mask, np.zeros_like(mask)))
    else:
        print('Unknown color, using red as default.')
        layer = alpha * np.dstack((mask, np.zeros_like(mask), np.zeros_like(mask)))
    return layer


def slim_seeds(seeds, sliceId=2):
    slims = np.zeros_like(seeds)
    if sliceId == 0:
        for i in range(seeds.shape[0]):
            layer = seeds[i,:,:]
            labels = skimor.label(layer, neighbors=4, background=0) + 1
            n_labels = labels.max()
            for o in range(1,n_labels+1):
                centroid = np.round(skimea.regionprops(labels == o)[0].centroid)
                slims[i, centroid[0], centroid[1]] = 1
    return slims



# ! /usr/bin/python
# -*- coding: utf-8 -*-


# import funkcí z jiného adresáře
import sys
import os.path

# imcut_path =  os.path.join(path_to_script, "../../imcut/")
# sys.path.insert(0, imcut_path)
import unittest

import numpy as np
from nose.plugins.attrib import attr


from lisa import organ_segmentation
import imcut.dcmreaddata as dcmr
import lisa.dataset
import io3d.datasets


# nosetests tests/organ_segmentation_test.py:OrganSegmentationTest.test_create_iparams # noqa


class OrganSegmentationTest(unittest.TestCase):
    interactiveTest = False
    verbose = False

    def generate_data(self):

        img3d = (np.random.rand(30, 30, 30)*10).astype(np.int16)
        seeds = (np.zeros(img3d.shape)).astype(np.int8)
        segmentation = (np.zeros(img3d.shape)).astype(np.int8)
        segmentation[10:25, 4:24, 2:16] = 1
        img3d = img3d + segmentation*20
        seeds[12:18, 9:16, 3:6] = 1
        seeds[19:22, 21:27, 19:21] = 2

        voxelsize_mm = [5, 5, 5]
        metadata = {'voxelsize_mm': voxelsize_mm}
        return img3d, metadata, seeds, segmentation

    # @unittest.skipIf(not interactiveTest, "interactive test")
    @attr("interactive")
    def test_viewer_seeds(self):

        try:
            from imcut.seed_editor_qt import QTSeedEditor
        except:
            print("Deprecated of pyseg_base as submodule")
            from seed_editor_qt import QTSeedEditor
        from PyQt4.QtGui import QApplication
        import numpy as np
        img3d = (np.random.rand(30, 30, 30)*10).astype(np.int16)
        seeds = (np.zeros(img3d.shape)).astype(np.int8)
        seeds[3:6, 12:18, 9:16] = 1
        seeds[3:6, 19:22, 21:27] = 2
        # , QMainWindow
        app = QApplication(sys.argv)
        pyed = QTSeedEditor(img3d, seeds=seeds)
        pyed.exec_()

        # deletemask = pyed.getSeeds()
        # import pdb; pdb.set_trace()

        # pyed = QTSeedEditor(deletemask, mode='draw')
        # pyed.exec_()

        app.exit()
    # @unittest.skip("demonstrating skipping")

    @attr("interactive")
    def test_whole_organ_segmentation_interactive(self):
        """
        Interactive test uses dicom data for segmentation
        """
        dcmdir = io3d.datasets.join_path(
            'matlab/examples/sample_data/DICOM/digest_article/'
        )
            # path_to_script,
            # './../sample_data/matlab/examples/sample_data/DICOM/digest_article/') # noqa
        oseg = organ_segmentation.OrganSegmentation(
            dcmdir, working_voxelsize_mm=4, manualroi=False)

# manual seeds setting
        print("with left mouse button select some pixels of the brain")
        print("with right mouse button select some pixels of other tissues\
and background")

        oseg.interactivity()

        volume = oseg.get_segmented_volume_size_mm3()
        print(volume)

        self.assertGreater(volume, 50000)
        self.assertLess(volume, 1200000)


#        roi_mm = [[3, 3, 3], [150, 150, 50]]
#        oseg.ni_set_roi()
#        coordinates_mm = [[110, 50, 30], [10, 10, 10]]
#        label = [1, 2]
#        radius = [5, 5]
#        oseg.ni_set_seeds(coordinates_mm, label, radius)
#
#        oseg.make_segmentation()


# @TODO doladit boundary penalties
    # @unittest.skipIf(not interactiveTest, "interactive test")
    @unittest.skip("interactivity params are obsolete")
    def test_organ_segmentation_with_boundary_penalties(self):
        """
        Interactivity is stored to file
        """
        dcmdir = io3d.datasets.join_path('jatra_5mm')

        print("Interactive test: with left mouse button select liver, \
            with right mouse button select other tissues")
        # gcparams = {'pairwiseAlpha':10, 'use_boundary_penalties':True}
        segparams = {'pairwise_alpha_per': 3,
                     'use_boundary_penalties': True,
                     'boundary_penalties_sigma': 200}
        oseg = organ_segmentation.OrganSegmentation(
            dcmdir, working_voxelsize_mm=4, segparams=segparams)
        oseg.add_seeds_mm([120], [120], [400], label=1, radius=30)
        oseg.add_seeds_mm([170, 220, 250], [250, 280, 200], [400], label=2,
                          radius=30)

        "boundary penalties"
        oseg.interactivity()
        # oseg.ninteractivity()

        volume = oseg.get_segmented_volume_size_mm3()

        # misc.obj_to_file(oseg.get_iparams(),'iparams.pkl', filetype='pickle')

        self.assertGreater(volume, 1000000)
    # @unittest.skipIf(not interactiveTest, "interactive test")

    @unittest.skip("interactivity params are obsolete")
    def test_create_iparams(self):
        """
        Interactivity is stored to file
        """
        if self.verbose:
            print("test_create_iparams")
        import misc
        dcmdir = io3d.datasets.join_path('jatra_5mm')
            # path_to_script, './../sample_data/jatra_5mm')

        segparams = {'pairwiseAlpha': 20,
                     'use_boundary_penalties': False,
                     'boundary_penalties_sigma': 200}
        # oseg = organ_segmentation.OrganSegmentation(dcmdir, working_voxelsize_mm = 4) # noqa
        oseg = organ_segmentation.OrganSegmentation(
            dcmdir, working_voxelsize_mm=4,
            segparams=segparams, manualroi=False)
        # oseg.add_seeds_mm([120, 160], [150, 120], [70], label=1, radius=20)
        oseg.add_seeds_mm([120, 160], [150, 80], [85], label=1, radius=20)
        oseg.add_seeds_mm([170, 220, 250, 100], [250, 300, 200, 350], [85],
                          label=2, radius=20)
        oseg.add_seeds_mm([170], [240], [70], label=2, radius=20)

        # print "test_ipars"
        # oseg.interactivity()
        oseg.ninteractivity()

        volume = oseg.get_segmented_volume_size_mm3()
        # print 'vol %.3g ' %(volume)

        misc.obj_to_file(oseg.get_iparams(), 'iparams.pkl', filetype='pickle')

        self.assertGreater(volume, 1000000)

    # @unittest.skipIf(not interactiveTest, "interactive test")
    @attr("interactive")
    def test_stored_interactivity(self):
        pass

# TODO finish this test
    @attr("interactive")
    def test_synth_liver(self):
        params = {}
        self.synthetic_liver_template(params)

    def synthetic_liver_template(self, params):
        """
        Function uses organ_segmentation  for synthetic box object
        segmentation.
        """
        # dcmdir = os.path.join(path_to_script,'./../sample_data/matlab/examples/sample_data/DICOM/digest_article/') # noqa
# data

        data3d, segm, voxelsize_mm, slab, seeds_liver, seeds_porta = io3d.datasets.generate_synthetic_liver()

# seeds
#         seeds = np.zeros(data3d.shape, np.int8)
#         seeds[40:55, 90:120, 70:110] = 1
#         seeds[30:45, 190:200, 40:90] = 2
# [mm]  10 x 10 x 10        # voxelsize_mm = [1, 4, 3]
        metadata = {'voxelsize_mm': voxelsize_mm}

        oseg = organ_segmentation.OrganSegmentation(
            None,
            data3d=data3d,
            metadata=metadata,
            seeds=seeds_liver,
            working_voxelsize_mm=5,
            manualroi=False,
            autocrop=False,

            **params
        )

        oseg.ninteractivity()

        volume = oseg.get_segmented_volume_size_mm3("liver")
        self.assertGreater(volume, 570000, "Liver volume should have proper size")
        self.assertLess(volume, 640000, "Liver volume should have proper size")

        oseg.portalVeinSegmentation(interactivity=False, threshold=180)

        volume_porta = oseg.get_segmented_volume_size_mm3("porta")
        self.assertGreater(volume_porta, 9000, "Liver volume should have proper size")
        self.assertLess(volume_porta, 11000, "Liver volume should have proper size")
        # import sed3
        # ed = sed3.sed3(data3d,
        #                contour=(oseg.segmentation))
        # ed.show()
        oseg.saveVesselTree('porta')

        # print '> 0 '
        # print np.sum(oseg.segmentation > 0)
        # print np.sum(segm > 0)
        # print np.sum(oseg.segmentation > 0) * np.prod(voxelsize_mm)
        # print np.sum(segm > 0) * np.prod(voxelsize_mm)
        # print 'computed ', volume
        # print voxelsize_mm
        # print oseg.voxelsize_mm

        # import pdb; pdb.set_trace()
        # import ipdb; ipdb.set_trace() #  noqa BREAKPOINT

        # mel by to být litr. tedy milion mm3
        # je to zvlastni. pro nekter verze knihoven je to 630, pro jine 580

    def test_roi(self):
        """
        Test setting of ROI. It is in pixels, not in mm
        """

        img3d = (np.random.rand(30, 30, 30)*10).astype(np.int16)
        seeds = (np.zeros(img3d.shape)).astype(np.int8)
        segmentation = (np.zeros(img3d.shape)).astype(np.int8)
        segmentation[10:25, 4:24, 2:16] = 1
        img3d = img3d + segmentation*20
        seeds[12:18, 9:16, 3:6] = 1
        seeds[19:22, 21:27, 19:21] = 2

        roi = [[7, 27], [2, 29], [0, 26]]
        # seeds = seeds[7:27, 2:29, 0:26]
        voxelsize_mm = [5, 5, 5]
        metadata = {'voxelsize_mm': voxelsize_mm}

        oseg = organ_segmentation.OrganSegmentation(
            None,
            data3d=img3d,
            metadata=metadata,
            seeds=seeds,
            roi=roi,
            working_voxelsize_mm=5,
            manualroi=False)

        # from PyQt4.QtGui import QApplication
        # app = QApplication(sys.argv)
        # oseg.interactivity(min_val=0, max_val=30)
        oseg.ninteractivity()
        datap = oseg.export()

        volume = oseg.get_segmented_volume_size_mm3()
        self.assertGreater(volume, 500000)
        self.assertIn('data3d', datap.keys())
        self.assertIn('voxelsize_mm', datap.keys())

    def test_box_segmentation(self):
        params = {'segmentation_smoothing': False}
        self.box_segmentation_template(params)

    def test_box_segmentation_with_smoothing(self):
        """
        Function uses organ_segmentation  for synthetic box object
        segmentation.
        """
        params = {'segmentation_smoothing': True}
        self.box_segmentation_template(params, noise_sigma=4)
        # dcmdir = os.path.join(path_to_script,'./../sample_data/matlab/examples/sample_data/DICOM/digest_article/') # noqa

    def box_segmentation_template(self, params, noise_sigma=3):
        """
        Function uses organ_segmentation  for synthetic box object
        segmentation.
        """
        # dcmdir = os.path.join(path_to_script,'./../sample_data/matlab/examples/sample_data/DICOM/digest_article/') # noqa
# data
        img3d = np.random.rand(32, 64, 64) * noise_sigma
        img3d[4:24, 12:32, 5:25] = img3d[4:24, 12:32, 5:25] + 30

# seeds
        seeds = np.zeros([32, 64, 64], np.int8)
        seeds[9:12, 13:29, 18:25] = 1
        seeds[9:12, 4:9, 3:32] = 2
# [mm]  10 x 10 x 10        # voxelsize_mm = [1, 4, 3]
        voxelsize_mm = [5, 5, 5]
        metadata = {'voxelsize_mm': voxelsize_mm}

        oseg = organ_segmentation.OrganSegmentation(
            None,
            data3d=img3d,
            metadata=metadata,
            seeds=seeds,
            working_voxelsize_mm=10,
            manualroi=False,
            **params
        )

        # oseg.seeds = seeds
        # oseg.make_gc()
# manual seeds setting

        from PyQt4.QtGui import QApplication
        app = QApplication(sys.argv)
        # oseg.interactivity()
        oseg.ninteractivity()

        volume = oseg.get_segmented_volume_size_mm3()

        # import pdb; pdb.set_trace()

        # mel by to být litr. tedy milion mm3
        self.assertGreater(volume, 800000)
        self.assertLess(volume, 1100000)

    def test_volume_resize(self):
        # from scipy.sparse.import lil_matrix

        pass

    # @unittest.skipIf(True, "interactive test")
    # @unittest.skipIf(not interactiveTest, "interactive test")
    @attr("interactive")
    def test_vincentka_06_slice_thickness_interactive(self):
        """
        Interactive test. SliceThickness is not voxel depth. If it is, this
        test will fail.
        """
        # dcmdir = os.path.join(path_to_script, './../sample_data/matlab/examples/sample_data/DICOM/digest_article/') #noqa
        dcmdir = os.path.expanduser(
            '~/data/medical/data_orig/vincentka/13021610/10200000/')
        dcmdir = os.path.expanduser(
            '~/data/medical/data_orig/vincentka/13021610/12460000/')
        oseg = organ_segmentation.OrganSegmentation(dcmdir,
                                                    working_voxelsize_mm=4,
                                                    manualroi=False)

# manual seeds setting
        print(
            "with left mouse button select some pixels of the bottle content")
        print("with right mouse button select some pixels of background")

        oseg.interactivity()

        volume = oseg.get_segmented_volume_size_mm3()
        # print volume

        self.assertGreater(volume, 550000)
        self.assertLess(volume, 850000)

    def setUp(self):
        """ Nastavení společných proměnných pro testy  """
        self.assertTrue(True)

    # @TODO dodělat přidávání uzlů pomocí mm
    # @unittest.skipIf(not interactiveTest, "interactive test")
    def test_add_seeds_mm(self):
        """
        Function uses organ_segmentation object for segmentation
        """
        dcmdir = io3d.datasets.join_path(
            'matlab/examples/sample_data/DICOM/digest_article/'
            # path_to_script,
            # './../sample_data/matlab/examples/sample_data/DICOM/digest_article/'
        )
        oseg = organ_segmentation.OrganSegmentation(dcmdir,
                                                    working_voxelsize_mm=4,
                                                    manualroi=False)

        oseg.add_seeds_mm([80], [120], [120], 1, 25)
        oseg.add_seeds_mm([80], [40], [130], 2, 25)

        # pro kontrolu lze odkomentovat
        # from PyQt4.QtGui import QApplication
        # app = QApplication(sys.argv)
        # oseg.interactivity()

        oseg.ninteractivity()

        volume = oseg.get_segmented_volume_size_mm3()

        # import pdb; pdb.set_trace()

        # mel by to být litr. tedy milion mm3
        self.assertGreater(volume, 800000)
        self.assertLess(volume, 1200000)

        # roi_mm = [[3, 3, 3], [150, 150, 50]]
        # oseg.ni_set_roi()
        # coordinates_mm = [[110, 50, 30], [10, 10, 10]]
        # label = [1, 2]
        # radius = [5, 5]
        # oseg.ni_set_seeds(coordinates_mm, label, radius)

        # oseg.make_segmentation()

        # oseg.noninteractivity()
        pass

    @unittest.skip("demonstrating skipping")
    def test_dicomread_and_graphcut(self):
        """
        Test dicomread module and graphcut module
        """
        from imcut import pycut

        path_to_script = os.path.dirname(os.path.abspath(__file__))
        dcmdir = os.path.join(path_to_script, './../sample_data/matlab/examples/sample_data/DICOM/digest_article/') #noqa
        data3d, metadata = dcmr.dcm_read_from_dir(dcmdir)

        # print("Data size: " + str(data3d.nbytes) + ', shape: ' + str(data3d.shape) ) #noqa

        igc = pycut.ImageGraphCut(data3d, zoom=0.5)
        seeds = igc.seeds
        seeds[0, :, 0] = 1
        seeds[60:66, 60:66, 5:6] = 2
        igc.noninteractivity(seeds)

        igc.make_gc()
        segmentation = igc.segmentation
        self.assertTrue(segmentation[14, 4, 1] == 0)
        self.assertTrue(segmentation[127, 120, 10] == 1)
        self.assertTrue(np.sum(segmentation == 1) > 100)
        self.assertTrue(np.sum(segmentation == 0) > 100)
        # igc.show_segmentation()

    def test_resize_data_one_value(self):
        datap = io3d.datasets.generate_abdominal()
        target_voxelsize_mm = 3
        oseg = organ_segmentation.OrganSegmentation(
                                                    working_voxelsize_mm=4,
                                                    manualroi=False)
        oseg.import_dataplus(dataplus=datap)
        orig_shape = oseg.data3d.shape

        oseg.resize_to_mm(target_voxelsize_mm)
        self.assertEqual(oseg.voxelsize_mm[0], target_voxelsize_mm)
        self.assertEqual(oseg.voxelsize_mm[1], target_voxelsize_mm)
        self.assertEqual(oseg.voxelsize_mm[2], target_voxelsize_mm)
        self.assertNotEqual(orig_shape[0], oseg.data3d.shape[0])
        self.assertNotEqual(orig_shape[1], oseg.data3d.shape[1])
        self.assertNotEqual(orig_shape[2], oseg.data3d.shape[2])

    def test_resize_data_three_values(self):
        datap = io3d.datasets.generate_abdominal()
        target_voxelsize_mm = [7.14, 4.1, 3]
        oseg = organ_segmentation.OrganSegmentation(
            working_voxelsize_mm=4,
            manualroi=False)
        oseg.import_dataplus(dataplus=datap)
        orig_shape = oseg.data3d.shape

        oseg.resize_to_mm(target_voxelsize_mm)
        self.assertEqual(oseg.voxelsize_mm[0], target_voxelsize_mm[0])
        self.assertEqual(oseg.voxelsize_mm[1], target_voxelsize_mm[1])
        self.assertEqual(oseg.voxelsize_mm[2], target_voxelsize_mm[2])
        self.assertNotEqual(orig_shape[0], oseg.data3d.shape[0])
        self.assertNotEqual(orig_shape[1], oseg.data3d.shape[1])
        self.assertNotEqual(orig_shape[2], oseg.data3d.shape[2])

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Module is used for converting data from pkl format to raw.
"""
import argparse
import numpy as np

import misc
import qmisc

def main():

    parser = argparse.ArgumentParser(description=__doc__) # 'Simple VTK Viewer')

    parser.add_argument('-i','--inputfile', default=None,
                      help='File as .pkl')
    parser.add_argument('-o','--outputfile', default=None,
                      help='Output file. Filetype is given by extension.')

    parser.add_argument('-k','--key', default='data3d',
                      help='Which key should be writen to output file. \
                        Default is "data3d". You can use "segmentation"')
    args = parser.parse_args()
    data = misc.obj_from_file(args.inputfile, filetype = 'pickle')
    data3d_uncrop = qmisc.uncrop(data[args.key], data['crinfo'], data['orig_shape'])
    #import ipdb; ipdb.set_trace() # BREAKPOINT
    import SimpleITK as sitk
    sitk_img = sitk.GetImageFromArray(data3d_uncrop.astype(np.uint16), isVector=True)
    sitk.WriteImage(sitk_img, args.outputfile)
    print("Warning: .mhd and .raw format has corupted metadta. You can edit it manually.")
if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
"""
Purpose:     (CZE-ZCU-FAV-KKY) Liver medical project

Author:      Pavel Volkovinsky
Email:       volkovinsky.pavel@gmail.com

Created:     2013/10/14
Copyright:   (c) Pavel Volkovinsky
"""

import segmentation as sg
import numpy as np

import misc

def main1():

    ## Vytvorim si vlastni testovaci matici.
    matrix = np.zeros((7, 7))
    matrix[0:2, 3:] = 0.7
    matrix[0:4, 0:2] = 0.8
    matrix[3, 2:4] = 1
    matrix[3:5, 4:6] = 1
    matrix[6, 3:5] = -0.7
    matrix[5, 4] = 1
    matrix[6, 6] = 1
    matrix[5:, 0:2] = 0
    matrix[3:5, 4] = 0
    print('Matrix:')
    print(matrix)

    matrixX = matrix.copy()
    matrixX[matrixX != 0] = 1
    print('Matrix binary:')
    print(matrixX)

    ## Vratit 4 nejvetsi objekty - v debug modu.
    obj = 2
    print('>>> Nejvetsi objekty v matici (' + str(obj) + '):')
    matrix2 = sg.get_priority_objects(matrix, nObj = obj, seeds = None, debug = True)

    ## Vytvoreni seedu - uhlopricka "/".
    print('>>> Moje seeds:')
    mySeeds = np.zeros((7, 7))
    for index in range(0, 7):
        mySeeds[6 - index, index] = 1
    print('mySeeds:')
    print(mySeeds)
    mySeeds = mySeeds.nonzero()
    print('mySeeds (nonzero):')
    print(mySeeds)

    ## Uplatneni seedu na celou matici
    print('>>> Seeds na celou matici:')
    matrix3 = sg.get_priority_objects(matrix, nObj = obj, seeds = mySeeds, debug = True)
    print('>>> Seeds na matici po vraceni nejvetsich objektu:')
    ## Uplatneni seedu na matici po ziskani nejvetsich objektu
    matrix4 = sg.get_priority_objects(matrix2, nObj = obj, seeds = mySeeds, debug = True)

def main2():

    slab = {'none':0, 'liver':1, 'porta':2}
    voxelsize_mm = np.array([1.0, 1.0, 1.2])

    segm = np.zeros([256, 256, 80], dtype = np.uint8)

    # liver
    segm[70:180, 40:190, 30:60] = slab['liver']
    # port
    segm[120:130, 70:190, 40:45] = slab['porta']
    segm[80:130, 100:110, 40:45] = slab['porta']
    segm[120:170, 130:135, 40:44] = slab['porta']

    data3d = np.zeros(segm.shape)
    data3d[segm == slab['liver']] = 156
    data3d[segm == slab['porta']] = 206
    #noise = (np.random.rand(segm.shape[0], segm.shape[1], segm.shape[2])*30).astype(np.int16)
    noise = (np.random.normal(0, 30, segm.shape))#.astype(np.int16)
    data3d = (data3d + noise).astype(np.uint8)

    # @TODO je tam bug, prohl??e? neum? korektn? pracovat s doubly
    #        app = QApplication(sys.argv)
    #        #pyed = QTSeedEditor(noise )
    #        pyed = QTSeedEditor(data3d)
    #        pyed.exec_()
    #        #img3d = np.zeros([256,256,80], dtype=np.int16)

    # pyed = sed3.sed3(data3d)
    # pyed.show()

    outputTmp = sg.vesselSegmentation(
        data3d,
        segmentation = segm == slab['liver'],
        voxelsize_mm = voxelsize_mm,
        threshold = 180,
        inputSigma = 0.15,
        dilationIterations = 2,
        nObj = 1,
        interactivity = True,
        biggestObjects = False,
        binaryClosingIterations = 5,
        binaryOpeningIterations = 1)

    if outputTmp == None:
       print('Final output is None')

def main3():

    import matplotlib
    matplotlib.use('TkAgg')

    from numpy import arange, sin, pi
    from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg, NavigationToolbar2TkAgg
    from matplotlib.figure import Figure

    import sys
    if sys.version_info[0] < 3:
        import Tkinter as Tk
    else:
        import tkinter as Tk

    def destroy(e): 
        
        sys.exit()

    root = Tk.Tk()
    root.wm_title("Embedding in TK")
    #root.bind("<Destroy>", destroy)


    f = Figure(figsize=(5,4), dpi=100)
    a = f.add_subplot(111)
    t = arange(0.0,3.0,0.01)
    s = sin(2*pi*t)

    a.plot(t,s)
    a.set_title('Tk embedding')
    a.set_xlabel('X axis label')
    a.set_ylabel('Y label')


    # a tk.DrawingArea
    canvas = FigureCanvasTkAgg(f, master=root)
    canvas.show()
    canvas.get_tk_widget().pack(side=Tk.TOP, fill=Tk.BOTH, expand=1)

    #toolbar = NavigationToolbar2TkAgg( canvas, root )
    #toolbar.update()
    canvas._tkcanvas.pack(side=Tk.TOP, fill=Tk.BOTH, expand=1)

    button = Tk.Button(master=root, text='Quit', command=sys.exit)
    button.pack(side=Tk.BOTTOM)

    Tk.mainloop()

def main4():

    data = misc.obj_from_file('c:\_bp_data\d5\org-liver-orig004.mhd-3mm_alpha45.pklz', filetype = 'pickle')

    outputTmp = sg.vesselSegmentation(
        data['data3d'],
        segmentation = data['segmentation'],
        threshold = -1,
        inputSigma = 0.15,
        dilationIterations = 2,
        interactivity = True,
        biggestObjects = False,
        binaryClosingIterations = 2,
        binaryOpeningIterations = 0)


if __name__ == '__main__':

    main4()



# /usr/bin/env python
# -*- coding: utf-8 -*-

import logging

logger = logging.getLogger(__name__)
import json
import numpy as np
import random
from scipy.spatial import Delaunay
from skimage import morphology
from collections import OrderedDict

description = {}


def get_segdata(json_data, datap, labels=None):
    Z = len(datap["segmentation"])
    X = len(datap["segmentation"][0])
    Y = len(datap["segmentation"][0][0])

    for slice in range(0, Z):
        nbr_drawings = json_data['drawings'][slice][0]['length']
        for draw in range(0, nbr_drawings):
            # zpracovavany label
            dict_key = json_data['drawingsDetails'][slice][0][draw]['textExpr']
            if labels != None and dict_key not in labels: # zpracovavani jen nekterych objektu
                continue

            # riznuti jsonu u souradnic krajnich bodu
            draw_info = json_data['drawings'][slice][0][str(draw)]
            start = draw_info.find("points")
            end = draw_info.find("stroke")
            points_data = draw_info[start + 9: end - 3].split(',')     

            # prepsani souradnic do pole
            len_array = int(len(points_data) / 2)
            points = np.empty((len_array, 2))
            i = 0
            for j in range(0, len_array):
                points[j] = [int(points_data[i + 1]), int(points_data[i])]
                i += 2

            # vyplneni nakresleneho obrazce
            # pri spatnem nakresleni krivky muze zde padnout
            hull = Delaunay(points)
            x, y = np.mgrid[0:X, 0:Y]
            grid = np.vstack([x.ravel(), y.ravel()]).T
            simplex = hull.find_simplex(grid)
            fill = grid[simplex >= 0, :]

            # vlozeni markeru do dat
            # pokud je label ve slovniku, pouzije se hodnota z nej, neprepise se na novou!
            # pokud label neni ve slovniku, priradi se mu nahodna hodnota v rozmezi 100 - 254, 
            # jestlize neobsahuje v popisu vlastni hodnotu
            # pokud neni uveden label, ale pouze hodnota, vytvori se label se strukturou "lbl_" + hodnota
            # jestlize neni uveden ani label, ani hodnota, nic se neprovede
            dict_value = 0
            dict_description = json_data['drawingsDetails'][slice][0][draw]['longText'].replace("'", '"')
            if dict_description == '':
                if dict_key == '':
                    print("Drawing is not defined at slice", slice)
                    continue
                else:
                    if dict_key in datap["slab"].keys():
                        dict_value = datap["slab"][dict_key]
                    else:
                        dict_value = random.randint(100, 254)
                        datap["slab"][dict_key] = dict_value
            else:
                dict_description = json.loads(dict_description)
                if dict_key != '' and dict_key in datap["slab"].keys():
                    dict_value = datap["slab"][dict_key]
                elif "value" in dict_description.keys():
                    dict_value = dict_description["value"]
                    if dict_key != '':
                        datap["slab"][dict_key] = dict_value
                    else:
                        dict_key = "lbl_" + str(dict_value)
                        datap["slab"][dict_key] = dict_value
                elif dict_key != '':
                    dict_value = random.randint(100, 254)
                    datap["slab"][dict_key] = dict_value
                else:
                    print("Drawing is not defined at slice", slice)
                    continue

            for i, j in fill:
                datap["segmentation"][Z - 1 - slice][i][j] = dict_value
        
            # ziskani zbytku popisu
            if dict_key not in description.keys():
                description[dict_key] = {}
                i_color = draw_info.find('#') + 1
                description[dict_key]["r"] = int(draw_info[i_color:(i_color + 2)], 16)
                description[dict_key]["g"] = int(draw_info[(i_color + 2):(i_color + 4)], 16)
                description[dict_key]["b"] = int(draw_info[(i_color + 4):(i_color + 6)], 16)
                description[dict_key]["value"] = dict_value

            if dict_description != '':
                if "threshold" in dict_description.keys():
                    description[dict_key]["threshold"] = dict_description["threshold"] # nastavit kolem 100 - 120
                if "two" in dict_description.keys():
                    description[dict_key]["two"] = dict_description["two"]
                if "three" in dict_description.keys():
                    description[dict_key]["three"] = dict_description["three"]


def get_vesselpoint_in_seeds(json_data, label, seeds_shape):
    seeds = np.zeros(seeds_shape, dtype=int)
    pts = get_vesselPoint(json_data, label)
    seeds[pts] = 1
    return seeds


def get_vesselPoint(json_data, label=None):
    Z = len(json_data['drawings'])

    if label==None:
        label = "porta"

    for slice in range(0, Z):
        nbr_drawings = json_data['drawings'][slice][0]['length']
        for draw in range(0, nbr_drawings):
            # zpracovavany label
            dict_key = json_data['drawingsDetails'][slice][0][draw]['textExpr']
            if dict_key != label:
                continue

            # riznuti jsonu u souradnic krajnich bodu
            draw_info = json_data['drawings'][slice][0][str(draw)]
            start = draw_info.find("points")
            end = draw_info.find("stroke")
            points_data = draw_info[start + 9: end - 3].split(',')     

            # prepsani souradnic do pole
            z = np.array([(Z - 1 - slice)], dtype=np.dtype('int64'))
            x = np.array([int(points_data[1])], dtype=np.dtype('int64'))
            y = np.array([int(points_data[0])], dtype=np.dtype('int64'))
            return [z, x, y]


def get_seeds(data, label):
    return ((data["segmentation"] != 0).astype('int8') * 2 - 
           (data["segmentation"] == data["slab"][label]).astype('int8'))


def write_to_json(datap, json_data=None, output_name="json_data.json"):
    Z = len(datap["segmentation"])
    X = len(datap["segmentation"][0])
    Y = len(datap["segmentation"][0][0])

    if json_data==None:
        json_data = initJson(Z) # prohazuje poradi => dwv nefunguje na 100 % (nelze pak stahnout json)

    for slice in range(0, Z):
        label_array = np.unique(datap["segmentation"][Z - 1 - slice]).tolist()
        label_array.remove(0) # pole vyskytovanych labelu bez 0

        divided = morphology.label(datap["segmentation"][Z - 1 - slice], background=0) # rozdeleni objektu
        nbr_divided = np.unique(divided).tolist() 
        nbr_divided.remove(0) # pole rozdelenych objektu bez 0
        logger.debug("slice, label array, number divided: {} {} {}".format(slice, label_array, nbr_divided))

        if len(label_array) < len(nbr_divided): # nesedi pocet vyskytovanych labelu s poctem objektu
            new_array = []
            for i in range(0, max(nbr_divided)):
                for j in range(0, len(label_array)):
                    a = ((datap["segmentation"] == label_array[j]) * (divided == i)).astype('int8')
                    if 1 in a:
                        new_array.append(label_array[j])
                        break
            label_array = new_array
        if len(label_array) > 0:
            nbr_drawings = json_data['drawings'][slice][0]['length']
            for lbl in range(0, len(label_array)):
                str_points = ""
                nbr_points = 0
                last_y = 0
                last_x = 0
                for key, value in datap["slab"].items(): # neni osetreno, kdyby se nenachazel label ve slovniku
                    if value != 0 and value == label_array[lbl]:
                        for x in range(0, X):
                            for y in range(0, Y):
                                if divided[x][y] == lbl + 1:
                                    str_points += ("," if len(str_points) != 0 else "") + str(y) + "," + str(x)
                                    last_y = y
                                    last_x = x
                                    nbr_points += 1
                        break
                if nbr_points <= 1: # osetreni, aby DWV nezamrzl (nesmi byt predan 1 stejny bod)
                    str_points += "," + str(last_y + 1) + "," + str(last_x)
                if key not in description.keys():
                    initDescription(key, b=0)

                rgba = "rgba(" + str(description[key]["r"])
                rgba += "," + str(description[key]["g"]) + ","
                rgba += str(description[key]["b"]) + ",0.5)"
                json_data["drawings"][slice][0][str(lbl + nbr_drawings)] = get_str_drawings(key + str(lbl), key, str_points, rgba, [150, 10 + lbl * 12])
                json_data["drawingsDetails"][slice][0].append({"id":key + str(lbl), "textExpr":key, "longText":"{\"value\":" + str(value) + "}", "quant":None})
            json_data["drawings"][slice][0]["length"] = len(label_array) + nbr_drawings
    with open(output_name, 'w') as file:
        json.dump(json_data, file)

def initJson(nbr_slices, window_center=50, window_width=350, y=0, x=0, z=0, scale=1):
    json_data = OrderedDict()
    json_data["version"] = "0.2"
    json_data["window-center"] = window_center
    json_data["window-width"] = window_width
    json_data["position"] = OrderedDict([("i", y), ("j", x), ("k", z)])
    json_data["scale"] = scale
    json_data["scaleCenter"] = OrderedDict([("x",0), ("y", 0)])
    json_data["translation"] = OrderedDict([("x",0), ("y", 0)])
    json_data["drawings"] = []
    json_data["drawingsDetails"] = []
    for slice in range(0, nbr_slices):
        json_data["drawings"].append([{"length":0}])
        json_data["drawingsDetails"].append([[]])
    return json_data

def initDescription(label, value=None, r=150, g=150, b=150):
    description[label] = {}
    description[label]["r"] = r
    description[label]["g"] = g
    description[label]["b"] = b
    if value==None:
        value = 93
    description[label]["value"] = value
    

def get_str_drawings(id, key, str_points, color, lbl_pos):
    string = "{\"attrs\":{\"name\":\"freeHand-group\",\"visible\":true,\"id\":\"" + id +"\"},"
    string += "\"className\":\"Group\",\"children\":[{\"attrs\":{\"points\":[" + str_points + "],"
    string += "\"stroke\":\"" + color + "\",\"strokeWidth\":2,\"name\":\"shape\",\"tension\":0.5,"
    string += "\"draggable\":true},\"className\":\"Line\"},"
    string += "{\"attrs\":{\"x\":" + str(lbl_pos[0]) + ",\"y\":" + str(lbl_pos[1]) + ",\"name\":\"label\"},"
    string += "\"className\":\"Label\",\"children\":[{\"attrs\":{\"fontSize\":11,\"fontFamily\":\"Verdana\","
    string += "\"fill\":\"" + color + "\",\"name\":\"text\",\"text\":\"" + key + "\"},"
    string += "\"className\":\"Text\"},{\"attrs\":{\"width\":24,\"height\":12},\"className\":\"Tag\"}]}]}"
    return string


"""PyAudio Example: Play a WAVE file."""



import math
import sys
import os.path


path_to_script = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.join(path_to_script, "../sound/"))


CHUNK = 1024

def play(filename):# pragma: no cover
# sudo apt-get install python-pyaudio
    import pyaudio
    import wave

    wf = wave.open(filename, 'rb')

    p = pyaudio.PyAudio()

    stream = p.open(format=p.get_format_from_width(wf.getsampwidth()),
                    channels=wf.getnchannels(),
                    rate=wf.getframerate(),
                    output=True)

    data = wf.readframes(CHUNK)

    while data != '':
        stream.write(data)
        data = wf.readframes(CHUNK)

    stream.stop_stream()
    stream.close()

    p.terminate()

def play2():# pragma: no cover
    import pyaudio
    import wave
    PyAudio = pyaudio.PyAudio
    RATE = 16000
    WAVE = 1000
    data = ''.join([chr(int(math.sin(x/((RATE/WAVE)/math.pi))*127+128)) for x in xrange(RATE)])
    p = PyAudio()

    stream = p.open(
            format=p.get_format_from_width(1),
            channels=1,
            rate=RATE,
            output=True
            )
    for DISCARD in xrange(5):
            stream.write(data)
            stream.stop_stream()
            stream.close()
            p.terminate()

def beep():# pragma: no cover
    #play2()
    #play(os.path.join(path_to_script, "../sound/beep-1.wav"))
    import subprocess
    from sys import platform
    if platform.startswith('linux'):
        subprocess.call('play -q -n synth 0.3 sine 880 vol 0.3', shell=True)
    else:
    #if platform == "win32":
        print('\7')

if __name__ == "__main__":
    if len(sys.argv) < 2:
        print("Plays a wave file.\n\nUsage: %s filename.wav" % sys.argv[0])
        sys.exit(-1)
    filename = sys.argv[1]
    beep()
    #play(filename)




#! /usr/bin/python
# -*- coding: utf-8 -*-
"""
Module is used for visualization of segmentation stored in pkl file.
"""

import sys
import os.path
path_to_script = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.join(path_to_script, "../extern/dicom2fem/src"))
import logging
logger = logging.getLogger(__name__)

# from PyQt4.QtCore import Qt
from PyQt4.QtGui import QApplication
import argparse


import numpy as np
import seg2fem
import misc
import viewer


def export_to_lar(segmentation, degrad):


    for xBlock in xrange(imageHeight/imageDx):
        # print "Working task: " +str(startImage) + "-" + str(endImage) + " [Xblock]"
        for yBlock in xrange(imageWidth/imageDy):
            # print "Working task: " +str(startImage) + "-" + str(endImage) + " [Yblock]"
            xStart, yStart = xBlock * imageDx, yBlock * imageDy
            xEnd, yEnd = xStart+imageDx, yStart+imageDy

            image = theImage[:, xStart:xEnd, yStart:yEnd]
            nz,nx,ny = image.shape

            # Compute a quotient complex of chains with constant field
            # ------------------------------------------------------------

            chains3D_old = [];
            chains3D = None
            hasSomeOne = False
            if (calculateout != True):
                chains3D = np.zeros(nx*ny*nz, dtype=np.int32)

            zStart = startImage - beginImageStack;

            if (calculateout == True):
                chains3D_old = cch.setList(nx,ny,nz, colorIdx, image,saveTheColors)
            else:
                hasSomeOne,chains3D = cch.setListNP(nx,ny,nz, colorIdx, image,saveTheColors)

            # print "Working task: " +str(startImage) + "-" + str(endImage) + " [hasSomeOne: " + str(hasSomeOne) +"]"

            # Compute the boundary complex of the quotient cell
            # ------------------------------------------------------------
            objectBoundaryChain = None
            if (calculateout == True) and (len(chains3D_old) > 0):
                objectBoundaryChain = larBoundaryChain(bordo3,chains3D_old)

            # Save
            if (calculateout == True):
                if (objectBoundaryChain != None):
                    writeOffsetToFile( fileToWrite, np.array([zStart,xStart,yStart], dtype=int32) )
                    fileToWrite.write( bytearray( np.array(objectBoundaryChain.toarray().astype('b').flatten()) ) )
            else:
                    if (hasSomeOne != False):
                            writeOffsetToFile( fileToWrite, np.array([zStart,xStart,yStart], dtype=int32) )
                            fileToWrite.write( bytearray( np.array(chains3D, dtype=np.dtype('b')) ) )

def showSegmentation(
        segmentation,
        voxelsize_mm=np.ones([3, 1]),
        degrad=4,
        label=1,
        smoothing=True
        ):
    """
    Funkce vrací trojrozměrné porobné jako data['segmentation']
    v data['slab'] je popsáno, co která hodnota znamená
    """
    labels = []

    segmentation = segmentation[::degrad, ::degrad, ::degrad]

    # import pdb; pdb.set_trace()
    mesh_data = seg2fem.gen_mesh_from_voxels_mc(segmentation, voxelsize_mm*degrad)
    if smoothing:
        mesh_data.coors = seg2fem.smooth_mesh(mesh_data)
    else:
        mesh_data = seg2fem.gen_mesh_from_voxels_mc(segmentation, voxelsize_mm * 1.0e-2)
        # mesh_data.coors +=
    vtk_file = "mesh_geom.vtk"
    mesh_data.write(vtk_file)
    QApplication(sys.argv)
    view = viewer.QVTKViewer(vtk_file)
    view.exec_()

    return labels

if __name__ == "__main__":
    logger = logging.getLogger()

    logger.setLevel(logging.WARNING)
    ch = logging.StreamHandler()
    logger.addHandler(ch)

    # logger.debug('input params')

    # input parser
    parser = argparse.ArgumentParser(
        description='\
            3D visualization of segmentation\n\
            \npython show_segmentation.py\n\
            \npython show_segmentation.py -i resection.pkl -l 2 3 4 -d 4')
    parser.add_argument(
        '-i', '--inputfile',
        default='organ.pkl',
        help='input file')
    parser.add_argument(
        '-d', '--degrad', type=int,
        default=4,
        help='data degradation, default 4')
    parser.add_argument(
        '-l', '--label', type=int, metavar='N', nargs='+',
        default=[1],
        help='segmentation labels, default 1')
    args = parser.parse_args()

    data = misc.obj_from_file(args.inputfile, filetype='pickle')
    # args.label = np.array(eval(args.label))
    # print args.label
    # import pdb; pdb.set_trace()
    ds = np.zeros(data['segmentation'].shape, np.bool)
    for i in range(0, len(args.label)):
        ds = ds | (data['segmentation'] == args.label[i])

    export_to_lar(ds, args.degrad)

    showSegmentation(ds, degrad=args.degrad)



#! /opt/local/bin/python
# -*- coding: utf-8 -*-

""" Run Lisa. """

# import funkcí z jiného adresáře
# import sys
# import os.path

# path_to_script = os.path.dirname(os.path.abspath(__file__))
# sys.path.append(os.path.join(path_to_script, "./src"))

if __name__ == "__main__":

    import lisa.organ_segmentation as osegg
    osegg.main()
    exit()



#! /usr/bin/env python
# -*- coding: utf-8 -*-

import unittest
from nose.plugins.attrib import attr
import numpy as np

import os.path as op
from lisa import organ_localizator, organ_model
import lisa.dataset
import io3d
import io3d.datareader


class LocalizationTests(unittest.TestCase):
    @attr('slow')
    def test_training(self):
        sample_data_path = lisa.dataset.sample_data_path()
        organ_localizator.train_liver_localizator_from_sliver_data(
            "liver.ol.p",
            sliver_reference_dir=sample_data_path,
            orig_pattern="*orig*001.mhd",
            ref_pattern="*seg*001.mhd",

        )

        ol = organ_localizator.OrganLocalizator()
        ol.load("liver.ol.p")

        dr = io3d.DataReader()
        data3d, metadata = dr.Get3DData(op.join(sample_data_path , 'liver-orig001.mhd'))
        out = ol.predict(data3d, metadata['voxelsize_mm'])


        seg, metadata = dr.Get3DData(op.join(sample_data_path , 'liver-seg001.mhd'))

        err = np.sum(np.abs(seg-out))
        # import sed3
        # sed3.show_slices(out-seg, out-seg, slice_step=20)
        # self.assertEqual(True, False)
        # less then 10% error expected
        self.assertGreater(np.prod(seg.shape)*0.1, err)

    @attr('slow')
    @attr('actual')
    def test_trainin_kidney_from_pklz(self):

        sample_data_path = lisa.dataset.join_sdp("kidney_training/")
        test_data = lisa.dataset.join_sdp("kidney_training/liver-orig001.mhd.pklz")
        label = "left kidney"


        organ_model.train_organ_model_from_dir(
            "left_kidney.ol.p",
            reference_dir=sample_data_path,
            orig_pattern="*orig*.pklz",
            ref_pattern="*orig*.pklz",
            label=label,
            segmentation_key=True
        )

        ol = organ_localizator.OrganLocalizator()
        ol.load("left_kidney.ol.p")

        dr = io3d.DataReader()
        data3d, metadata = dr.Get3DData(test_data, dataplus_format=False)
        out = ol.predict(data3d, metadata['voxelsize_mm'])

        seg = (metadata["segmentation"] == metadata['slab'][label]).astype(np.int8)

        err = np.sum(np.abs(seg-out))
        import sed3
        sed3.show_slices(out-seg, out-seg, slice_step=20)
        # self.assertEqual(True, False)
        # less then 10% error expected
        self.assertGreater(np.prod(seg.shape)*0.1, err)

if __name__ == '__main__':
    unittest.main()



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim:fenc=utf-8
#
# Copyright © %YEAR%  <>
#
# Distributed under terms of the %LICENSE% license.

"""
Training module. Default setup makes nothing. Use --all to make all
"""

import logging

logger = logging.getLogger(__name__)
import argparse


import organ_localizator
import organ_model

def main():
    logger = logging.getLogger()

    logger.setLevel(logging.DEBUG)
    ch = logging.StreamHandler()
    logger.addHandler(ch)

    # create file handler which logs even debug messages
    # fh = logging.FileHandler('log.txt')
    # fh.setLevel(logging.DEBUG)
    # formatter = logging.Formatter(
    #     '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    # fh.setFormatter(formatter)
    # logger.addHandler(fh)
    # logger.debug('start')

    # input parser
    parser = argparse.ArgumentParser(
            description=__doc__
    )
    parser.add_argument(
            '-i', '--inputfile',
            default=None,
            # required=True,
            help='input file'
    )
    parser.add_argument(
            '-d', '--debug', action='store_true',
            help='Debug mode')
    parser.add_argument(
            '-lm', '--liver-model', action='store_true',
            help='Train liver model')
    parser.add_argument(
            '-ll', '--liver-localizator', action='store_true',
            help='Train liver localizator')
    parser.add_argument(
            '--all', action='store_true',
            help='Train all')
    args = parser.parse_args()

    if args.debug:
        ch.setLevel(logging.DEBUG)


    if args.liver_localizator or args.all:
        organ_localizator.train_liver_localizator_from_sliver_data()


    if args.liver_model or args.all:
        organ_model.train_liver_model_from_sliver_data()


if __name__ == "__main__":
    main()


#! /usr/bin/python
# -*- coding: utf-8 -*-
import os
import zipfile
import subprocess
import sys
# import traceback

import logging
logger = logging.getLogger(__name__)

import argparse

if sys.version_info < (3, 0):
    import urllib as urllibr
else:
    import urllib.request as urllibr


# import funkcí z jiného adresáře
import os.path

path_to_script = os.path.dirname(os.path.abspath(__file__))


def submodule_update():
    # update submodules codes
    print('Updating submodules')
    try:
        # import pdb; pdb.set_trace()
        subprocess.call('git submodule update --init --recursive', shell=True)
        # subprocess.call('git submodule update --init --recursive')

    except:
        print('Probem with git submodules')


def check_python_architecture(pythondir, target_arch_str):
    """
    functions check architecture of target python
    """
    pyth_str = subprocess.check_output(
        [pythondir + 'python', '-c',
         'import platform; print(platform.architecture()[0])'])
    if pyth_str[:2] != target_arch_str:
        raise Exception(
            "Wrong architecture of target python. Expected arch is"
            + target_arch_str)


def definitions_win64_py32():
    global urlpython, urlmsysgit, urlmingw, urlnumpy, urlscipy, urlsklearn,\
        urlmatplotlib, urlcython, urlgco_python, urlgco, urlcompiler,\
        pythondir, pythonversion, target_arch_str
    urlpython = "http://www.python.org/ftp/python/3.2.3/python-3.2.3.amd64.msi"
    urlmsysgit = "http://msysgit.googlecode.com/files/Git-1.8.0-preview20121022.exe"
    urlmingw = "http://downloads.sourceforge.net/project/mingw/Installer/mingw-get-inst/mingw-get-inst-20120426/mingw-get-inst-20120426.exe?r=http%3A%2F%2Fsourceforge.net%2Fprojects%2Fmingw%2Ffiles%2Flatest%2Fdownload%3Fsource%3Dfiles&ts=1359726876&use_mirror=ignum",
    urlnumpy = "http://home.zcu.cz/~mjirik/liver/download/win64_py32/numpy-MKL-1.6.2.win-amd64-py3.2.exe"
    urlscipy = "http://home.zcu.cz/~mjirik/liver/download/win64_py32/scipy-0.11.0.win-amd64-py3.2.exe"
    urlsklearn = "http://home.zcu.cz/~mjirik/liver/download/win64_py32/scikit-learn-0.13.win-amd64-py3.2.exe"
    urlmatplotlib = "http://home.zcu.cz/~mjirik/liver/download/win64_py32/matplotlib-1.2.0.win-amd64-py3.2.exe"
    urlcython = "http://home.zcu.cz/~mjirik/liver/download/win64_py32/Cython-0.18.win-amd64-py3.2.exe"
    urlgco_python = "https://github.com/amueller/gco_python/archive/master.zip"
    urlgco = "http://vision.csd.uwo.ca/code/gco-v3.0.zip"
    urlcompiler = "http://home.zcu.cz/~mjirik/liver/download/win64_py32/cygwinccompiler.py"
    pythondir = "c:/python32/"
    pythonversion = (3, 2)
    target_arch_str = '64'


def definitions_win32_py27():
    global urlpython, urlmsysgit, urlmingw, urlnumpy, urlscipy, urlsklearn, urlmatplotlib, urlcython, urlgco_python, urlgco, urlcompiler, pythondir, pythonversion, target_arch_str
    urlpython = "http://www.python.org/ftp/python/2.7.3/python-2.7.3.msi"
    urlmsysgit = "http://msysgit.googlecode.com/files/Git-1.8.0-preview20121022.exe"
    urlmingw = "http://downloads.sourceforge.net/project/mingw/Installer/mingw-get-inst/mingw-get-inst-20120426/mingw-get-inst-20120426.exe?r=http%3A%2F%2Fsourceforge.net%2Fprojects%2Fmingw%2Ffiles%2Flatest%2Fdownload%3Fsource%3Dfiles&ts=1359726876&use_mirror=ignum"
    urlnumpy = "http://home.zcu.cz/~mjirik/liver/download/win32_py27/numpy-MKL-1.6.2.win32-py2.7.exe"
    urlscipy = "http://home.zcu.cz/~mjirik/liver/download/win32_py27/scipy-0.11.0.win32-py2.7.exe"
    urlsklearn = "http://home.zcu.cz/~mjirik/liver/download/win32_py27/scikit-learn-0.13.win32-py2.7.exe"
    urlmatplotlib = "http://home.zcu.cz/~mjirik/liver/download/win32_py27/matplotlib-1.2.0.win32-py2.7.exe"
    urlcython = "http://home.zcu.cz/~mjirik/liver/download/win32_py27/Cython-0.17.4.win32-py2.7.exe"
    urlgco_python = "https://github.com/amueller/gco_python/archive/master.zip"
    urlgco = "http://vision.csd.uwo.ca/code/gco-v3.0.zip"
    urlcompiler = "http://home.zcu.cz/~mjirik/liver/download/win32_py27/cygwinccompiler.py"
    pythondir = "c:/python27/"
    pythonversion = (2,7)
    target_arch_str = '32'


def windows_install():
    try:
        os.mkdir("tmp")
    except:
        pass

    # problems with 64 bit version and mingw
    #definitions_win64_py32()
    definitions_win32_py27()

    # check correct python version
    if (sys.version_info[0:2] != pythonversion):

        print("Different python required \n installing python "
               + str(pythonversion))
        print(sys.version_info)

        local_file_name = 'tmp\python.msi'
        urllibr.urlretrieve(urlpython, local_file_name)
        a = "msiexec /i " + local_file_name
        print(a)
        subprocess.call(a)

        check_python_architecture(pythondir, target_arch_str)

        print("Please restart installer with new python")
        subprocess.call(pythondir + "python.exe mysetup.py")
        return
    else:
        print("You have python " + str(pythonversion))

    # install distribute and pip
    # this is not necessary for numpy, scipy etc., because we have binaries
    # but we need it for pydicom
    urldistribute = "http://python-distribute.org/distribute_setup.py"
    local_file_name = "./tmp/distribute_setup.py"
    urllibr.urlretrieve(urldistribute, local_file_name)
    subprocess.call(pythondir + "python.exe distribute_setup.py", cwd="./tmp/")

    urlpip = "https://raw.github.com/pypa/pip/master/contrib/get-pip.py"
    local_file_name = "./tmp/get-pip.py"
    urllibr.urlretrieve(urlpip, local_file_name)
    subprocess.call(pythondir + "python.exe get-pip.py", cwd="./tmp/")

    # numpy, scipy, matplotlib, scikit-learn, cython
    print("numpy, scipy, matplotlib, scikit-learn, cython install")
    #import pdb; pdb.set_trace()
    try:

        subprocess.call(
            pythondir +
            "Scripts/pip.exe install numpy scipy scikit-learn"
            + " matplotlib cython nose",
            cwd="./tmp/")
    except:
        print("alternative installation ")
        download_and_run(urlnumpy, "./tmp/numpy.exe")
        download_and_run(urlscipy, "./tmp/scipy.exe")
        download_and_run(urlsklearn, "./tmp/sklearn.exe")
        download_and_run(urlmatplotlib, "./tmp/matplotlib.exe")
        download_and_run(urlcython, "./tmp/cython.exe")

    # install pydicom
    subprocess.call(pythondir + "Scripts/pip.exe install pydicom",
                    cwd="./tmp/")
    subprocess.call(pythondir + "Scripts/pip.exe install pyyaml", cwd="./tmp/")


def windows_get_git():

    # instalace msys gitu
    print("MSYS Git install")
    #import pdb; pdb.set_trace()
    download_and_run(url=urlmsysgit, local_file_name='./tmp/msysgit.exe')


def windows_build_gco():
    # install MinGW
    print("MinGW install")
    download_and_run(url=urlmingw, local_file_name='./tmp/mingw.exe')

    # install gco_python
    print("gco_python install")
    # --- first we need gco_python source codes
    #try:
    #    os.mkdir("tmp/gco_python")
    #except:
    #    pass

    local_file_name = './tmp/gco_python.zip'
    urllibr.urlretrieve(urlgco_python, local_file_name)
    datafile = zipfile.ZipFile(local_file_name)
    datafile.extractall('./tmp/')

    # --- second we need gco sources
    try:
        os.mkdir("tmp/gco_python-master/gco_src")
    except:
        pass

    local_file_name = './tmp/gco_src.zip'
    urllibr.urlretrieve(urlgco, local_file_name)
    datafile = zipfile.ZipFile(local_file_name)
    datafile.extractall('./tmp/gco_python-master/gco_src')

    # --- now compile
    print("gco compilation")

    # there is a bug in python mingw compiler with -mno-cygwin, so here is simple patch

    local_file_name = pythondir + "Lib\distutils\cygwinccompiler.py"
    urllibr.urlretrieve(urlcompiler, local_file_name)

    # compilation with mingw
    subprocess.call(pythondir + "python.exe setup.py build_ext -i --compiler=mingw32", cwd="./tmp/gco_python-master/")
    # parametr -i by to mel nainstalovat sam, ale nedala to
    subprocess.call(pythondir + "python.exe setup.py build --compiler=mingw32", cwd="./tmp/gco_python-master/")
    subprocess.call(pythondir + "python.exe setup.py install --skip-build", cwd="./tmp/gco_python-master/")

    #subprocess.call('envinstall\envwindows.bat')


def remove(local_file_name):
    try:
        os.remove(local_file_name)
    except Exception as e:
        print("Cannot remove file '" + local_file_name + "'. Please remove\
        it manually.")
        print(e)


def downzip(url, destination='./sample_data/'):
    """
    Download, unzip and delete.
    """

    # url = "http://147.228.240.61/queetech/sample-data/jatra_06mm_jenjatra.zip"
    local_file_name = './sample_data/tmp.zip'
    urllibr.urlretrieve(url, local_file_name)
    datafile = zipfile.ZipFile(local_file_name)
    datafile.extractall(destination)
    remove(local_file_name)


def get_sample_data():
    """
    This function is deprecated. Now we use imtools.qmisc.get_sample_data
    :return:
    """
    # download sample data
    print('Downloading sample data')

    try:
        os.mkdir('sample_data')
    except:
        pass

    # Puvodni URL z mathworks
    # url =  "http://www.mathworks.com/matlabcentral/fileexchange/2762-dicom-example-files?download=true"
    # url = "http://www.mathworks.com/includes_content/domainRedirect/domainRedirect.html?uri=http%3A%2F%2Fwww.mathworks.com%2Fmatlabcentral%2Ffileexchange%2F2762-dicom-example-files%3Fdownload%3Dtrue%26nocookie%3Dtrue"
    url = "http://147.228.240.61/queetech/sample-data/head.zip"
    local_file_name = './sample_data/head.zip'

    urlobj = urllibr.urlopen(url)
    url = urlobj.geturl()
    urllibr.urlretrieve(url, local_file_name)

    datafile = zipfile.ZipFile(local_file_name)
    #datafile.setpassword('queetech')
    datafile.extractall('./sample_data/')
    remove(local_file_name)

# get jatra_06mm_jenjatra

    # url = "http://147.228.240.61/queetech/sample-data/jatra_06mm_jenjatraplus.zip"
    # local_file_name = './sample_data/jatra_06mm_jenjatraplus.zip'
    url = "http://147.228.240.61/queetech/sample-data/jatra_06mm_jenjatra.zip"
    local_file_name = './sample_data/jatra_06mm_jenjatra.zip'
    urllibr.urlretrieve(url, local_file_name)
    datafile = zipfile.ZipFile(local_file_name)
    datafile.extractall('./sample_data/')
    remove(local_file_name)
# get jatra 5mm
    url = "http://147.228.240.61/queetech/sample-data/jatra_5mm.zip"
    local_file_name = './sample_data/jatra_5mm.zip'
    urllibr.urlretrieve(url, local_file_name)
    datafile = zipfile.ZipFile(local_file_name)
    datafile.extractall('./sample_data/')
    remove(local_file_name)

# get experiment data
    url = "http://147.228.240.61/queetech/sample-data/exp.zip"
    local_file_name = './sample_data/exp.zip'
    urllibr.urlretrieve(url, local_file_name)
    datafile = zipfile.ZipFile(local_file_name)
    datafile.extractall('./sample_data/')
    remove(local_file_name)
# get sliver sample
    url = "http://147.228.240.61/queetech/sample-data/sliver_training_001.zip"
    local_file_name = './sample_data/sliver_training_001.zip'
    urllibr.urlretrieve(url, local_file_name)
    datafile = zipfile.ZipFile(local_file_name)
    datafile.extractall('./sample_data/')
    remove(local_file_name)

# get volumetry sample
    url = "http://147.228.240.61/queetech/sample-data/volumetrie.zip"
    local_file_name = './sample_data/volumetrie.zip'
    urllibr.urlretrieve(url, local_file_name)
    datafile = zipfile.ZipFile(local_file_name)
    datafile.extractall('./sample_data/')
    remove(local_file_name)

# get organ.pkl and vessels.pkl
    url = "http://147.228.240.61/queetech/sample-data/organ.pkl.zip"
    local_file_name = './sample_data/organ.pkl.zip'
    urllibr.urlretrieve(url, local_file_name)
    datafile = zipfile.ZipFile(local_file_name)
    datafile.extractall('./sample_data/')
    remove(local_file_name)

    url = "http://147.228.240.61/queetech/sample-data/vessels.pkl.zip"
    local_file_name = './sample_data/vessels.pkl.zip'
    urllibr.urlretrieve(url, local_file_name)
    datafile = zipfile.ZipFile(local_file_name)
    datafile.extractall('./sample_data/')
    remove(local_file_name)

# get biodur samples
    url = "http://147.228.240.61/queetech/sample-data/biodur_sample.zip"
    local_file_name = './sample_data/biodur_sample.zip'
    urllibr.urlretrieve(url, local_file_name)
    datafile = zipfile.ZipFile(local_file_name)
    datafile.extractall('./sample_data/')
    remove(local_file_name)

# get gensei samples
    url = "http://147.228.240.61/queetech/sample-data/gensei_slices.zip"
    local_file_name = './sample_data/gensei_slices.zip'
    urllibr.urlretrieve(url, local_file_name)
    datafile = zipfile.ZipFile(local_file_name)
    datafile.extractall('./sample_data/')
    remove(local_file_name)

    downzip("http://147.228.240.61/queetech/sample-data/exp_small.zip")


def windows_get_gco():
    url = "http://147.228.240.61/queetech/install/pygco-py27-32bit/pygco.pyd"

    local_file_name = "C:\python27\Lib\site-packages\pygco.pyd"
    urllibr.urlretrieve(url, local_file_name)


def download_and_run(url, local_file_name):
    urllibr.urlretrieve(url, local_file_name)
    subprocess.call(local_file_name)


def get_conda_path():
    import os.path as op
    conda_pth = op.expanduser('~/anaconda/bin')
    if not op.exists(conda_pth):
        conda_pth = op.expanduser('~/miniconda/bin')
    return conda_pth


def file_copy_and_replace_lines(in_path, out_path):
    import shutil
    import fileinput

    # print "path to script:"
    # print path_to_script
    lisa_path = os.path.abspath(path_to_script)

    shutil.copy2(in_path, out_path)
    conda_path = get_conda_path()

    # print 'ip ', in_path
    # print 'op ', out_path
    # print 'cp ', conda_path
    for line in fileinput.input(out_path, inplace=True):
        # coma on end makes no linebreak
        line = line.replace("@{LISA_PATH}", lisa_path)
        line = line.replace("@{CONDA_PATH}", conda_path)
        print(line)


def make_icon():
    import platform

    system = platform.system()
    if system == 'Darwin':
        # MacOS
        __make_icon_osx()
        pass
    elif system == "Linux":
        __make_icon_linux()


def __make_icon_osx():
    home_path = os.path.expanduser('~')
    in_path = os.path.join(path_to_script, "applications/lisa_gui")
    dt_path = os.path.join(home_path, "Desktop")
    subprocess.call(['ln', '-s', in_path, dt_path])


def __make_icon_linux():

    in_path = os.path.join(path_to_script, "applications/lisa.desktop.in")
    in_path_ha = os.path.join(path_to_script, "applications/ha.desktop.in")
    print("icon input path:")
    print(in_path, in_path_ha)

    home_path = os.path.expanduser('~')

    if os.path.exists(os.path.join(home_path, 'Desktop')):
        desktop_path = os.path.join(home_path, 'Desktop')
    elif os.path.exists(os.path.join(home_path, 'Plocha')):
        desktop_path = os.path.join(home_path, 'Plocha')
    else:
        print("Cannot find desktop directory")
        desktop_path = None

    # copy desktop files to desktop
    if desktop_path is not None:
        out_path = os.path.join(desktop_path, "lisa.desktop")
        out_path_ha = os.path.join(desktop_path, "ha.desktop")

        # fi = fileinput.input(out_path, inplace=True)
        print("icon output path:")
        print(out_path, out_path_ha)
        file_copy_and_replace_lines(in_path, out_path)
        file_copy_and_replace_lines(in_path_ha, out_path_ha)

    # copy desktop files to $HOME/.local/share/applications/
    # to be accesable in application menu (Linux)
    local_app_path = os.path.join(home_path, '.local/share/applications')
    if os.path.exists(local_app_path) and os.path.isdir(local_app_path):
        out_path = os.path.join(local_app_path, "lisa.desktop")

        out_path_ha = os.path.join(local_app_path, "ha.desktop")

        print("icon output path:")
        print(out_path, out_path_ha)
        file_copy_and_replace_lines(in_path, out_path)
        file_copy_and_replace_lines(in_path_ha, out_path_ha)

    else:
        print("Couldnt find $HOME/.local/share/applications/.")


def main():
    logger = logging.getLogger()

    logger.setLevel(logging.WARNING)
    ch = logging.StreamHandler()
    logger.addHandler(ch)

    #logger.debug('input params')

    # input parser
    parser = argparse.ArgumentParser(
        description=
        'Segment vessels from liver \n \npython organ_segmentation.py\n \n\
        python organ_segmentation.py -mroi -vs 0.6')
    parser.add_argument(
        '-d', '--get_sample_data', action='store_true',
        default=False,
        help='Get sample data')
    parser.add_argument(
        '-i','--install', action='store_true',
        default=False,
        help='Install')
    parser.add_argument('-icn','--make_icon', action='store_true',
            default=False,
            help='Creates desktop icon, works only in ubuntu')
    parser.add_argument('-g','--get_git', action='store_true',
            default=False,
            help='Get git in windows')

    parser.add_argument('--build_gco', action='store_true',
            default = False, help='Build gco_python in windows. Problematic step.')
    args = parser.parse_args()

#    if args.get_sample_data == False and args.install == False and args.build_gco == False:
## default setup is install and get sample data
#        args.get_sample_data = True
#        args.install = True
#        args.build_gco = False

    if args.get_sample_data:
        import lisa
        import lisa.dataset
        lisa.dataset.get_sample_data()
        # get_sample_data()

    if args.make_icon:
        make_icon()

    if args.install:
        print('Installing system environment')
        if sys.platform.startswith('linux'):

            subprocess.call('./envinstall/envubuntu.sh')
            #submodule_update()
        elif sys.platform.startswith('win'):
            windows_install()
            if args.build_gco:
                windows_build_gco()
            else:
                windows_get_gco()
            if args.get_git:
                windows_get_git()
                #submodule_update()


if __name__ == "__main__":
    main()
